/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.berechtigung;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Klasse fr das Abfragen der Oberflchenberechtigung durch die Plugins.<br>
 * Hinweis: Diese Klasse implementiert das Singleton-Pattern und somit muss die Instanz mittels <code>getInstanz</code>
 * geholt werden.
 * 
 * Realisierung der technischen Anforderungen TBuV 103 bis 105.
 * 
 * @version $Revision: 1.2 $ / $Date: 2009/12/08 08:50:11 $ / ($Author: HofmannJ $)
 * 
 * @author HofmannJ
 * 
 */
public class OberflaechenBerechtigung
{
  /**
   * Instanzvariable dieses Singletons.
   */
  private static OberflaechenBerechtigung _instanz = null;

  /**
   * Liste aller registrierten Funktionen.
   */
  private List<OberflaechenFunktion> _funktionen = new LinkedList<OberflaechenFunktion>();

  /**
   * Liste der verwendeten SystemObjekte.
   */
  private List<SystemObject> _systemObjekte = new LinkedList<SystemObject>();

  private static DaVKommunikation _dav;

  /**
   * Liefert die einzigste Instanz dieses Singletons.
   * 
   * @return Einzigste Instanz dieses Singletons.
   */
  public static OberflaechenBerechtigung getInstanz()
  {
    if (_instanz == null)
    {
      _instanz = new OberflaechenBerechtigung();
    }
    return _instanz;
  }

  /**
   * Privater Konstruktor.
   * 
   */
  private OberflaechenBerechtigung()
  {
    _dav = DaVKommunikation.getInstanz();
    
    /*
     * Hier muss auf die DaVVerbindung mit den entsprechenden Listenern gehrt werden.
     */
    IDaVVerbindungZuhoerer zuhoerer = new IDaVVerbindungZuhoerer() {
      @SuppressWarnings("unchecked")
			public void verbindungAbgebaut(DaVVerbindungEreignis e)
      {
        
        List list = new ArrayList();
        OberflaechenBerechtigungEreignis oe = new OberflaechenBerechtigungEreignis(this, true, list, list, _funktionen, list);

        DaVKommunikation.getInstanz().benachrichtigeOberflaechenBenutzerZuhoerer(oe);
      }

      @SuppressWarnings("unchecked")
			public void verbindungAufgebaut(DaVVerbindungEreignis e)
      {
    
        List list = new ArrayList();
        OberflaechenBerechtigungEreignis oe = new OberflaechenBerechtigungEreignis(this, false, list, list, list, _funktionen);
        DaVKommunikation.getInstanz().benachrichtigeOberflaechenBenutzerZuhoerer(oe);
    
        _dav.neuAnmeldung(); 
        
      }

    };

    DaVVerbindung.getInstanz().hinzufuegenZuhoerer(zuhoerer);
    
    
  }

  /**
   * Fgt eine {@link OberflaechenFunktion} zur Liste der OberflaechenFunktionen hinzu.
   * 
   * @param funktion
   *          {@link OberflaechenFunktion}, die hinzugefgt werden soll.
   */
  public void hinzufuegenOberflaechenFunktion(OberflaechenFunktion funktion)
  {
    _funktionen.add(funktion);
  }

  /**
   * Entfernt eine {@link OberflaechenFunktion} aus der Liste der OberflaechenFunktionen.
   * 
   * @param funktion
   *          {@link OberflaechenFunktion}, die entfernt werden soll.
   */
  public void entferneOberflaechenFunktion(OberflaechenFunktion funktion)
  {
    _funktionen.remove(funktion);
  }

  /**
   * Prft fr alle Elemente, die sich unter ID einer Gruppe, die z.B. einen Dialog bezeichnet, registriert haben, ob
   * die registrierten Berechtigungen fr die bergebenen Systemobjekte bestehen.
   * 
   * @param systemObjekte
   *          Systemobjekte, fr die die Berechtigung geprft werden soll.
   * @param dialogId
   *          Id des Dialogs, desse Elementberechtigung fr die bergebenen Systemobjekte geprft werden soll.
   */
  public void pruefeBerechtigungElementGruppe(List<SystemObject> systemObjekte, String dialogId)
  {
    // Ereignis erzeugen + feuern
    OberflaechenBerechtigungEreignis ereignis = pruefeBerechtigung(systemObjekte, dialogId, true);

    feureEreignis(ereignis);
  }

  /**
   * Prft fr ein registriertes Element, ob fr die bergebenen Systemobjekte die Berechtigung besteht.
   * 
   * @param systemObjekte
   *          Systemobjekte, fr die die Berechtigung geprft werden soll.
   * @param elementId
   *          Id des Elements, dessen Berechtigung fr die bergebenen Systemobjekte geprft werden soll.
   */
  public void pruefeBerechtigungElement(List<SystemObject> systemObjekte, String elementId)
  {
    // Ereignis erzeugen und feuern.
    OberflaechenBerechtigungEreignis ereignis = pruefeBerechtigung(systemObjekte, elementId, false);

    feureEreignis(ereignis);

  }

  /**
   * Dieser Event wird dann abgefeuert, wenn sich etwas am Berechtigungskonzept gendert hat.
   * 
   */
  private void feureEreignis(OberflaechenBerechtigungEreignis ereignis)
  {
    _dav.benachrichtigeOberflaechenBenutzerZuhoerer(ereignis);
  }

  /**
   * Prft sowohl fr ein registriertes Element, als auch fr alle Elemente, die sich unter ID einer Gruppe, registriert
   * haben, ob fr die bergebenen Systemobjekte die Berechtigung besteht. Die Prfung erfolgt in Abhngigkeit von der
   * Flagge isDialogId.
   * 
   * @param systemObjekte
   *          Systemobjekte, fr die die Berechtigung geprft werden soll.
   * @param id
   *          Dialog- oder Element-Id des Elements, dessen Berechtigung fr die bergebenen Systemobjekte geprft werden
   *          soll.
   * @param isDialogId
   *          zeigt an, ob es sich um die Dialog-Id handelt
   * @return OberflaechenBerechtigungEreignis
   * 
   */
  private OberflaechenBerechtigungEreignis pruefeBerechtigung(List<SystemObject> systemObjekte, String id,
      boolean isDialogId)
  {

    _systemObjekte = systemObjekte;

    List<OberflaechenFunktion> funktionen = new ArrayList<OberflaechenFunktion>();
    for (OberflaechenFunktion f : _funktionen)
    {

      if (isDialogId)
      {
        if (id.equals(f.getDialogId()))
          funktionen.add(f);
      }
      else if (id.equals(f.getFunktionsName()) || id.equals(f.getFunktionsAlias()))
        funktionen.add(f);
    }

    List<SystemObject> soFreigabe = new ArrayList<SystemObject>();
    List<SystemObject> soSperrung = new ArrayList<SystemObject>();

    List<OberflaechenFunktion> ofFreigabe = new ArrayList<OberflaechenFunktion>();
    List<OberflaechenFunktion> ofSperrung = new ArrayList<OberflaechenFunktion>();

    boolean flag = true;
    
    if (_dav.getConnection() != null){
    
      DynamicObject user = _dav.getConnection().getLocalUser();
  
      Boolean[] flags = new Boolean[systemObjekte.size()];
  
      for (OberflaechenFunktion f : funktionen)
      {
        for (int i = 0; i < systemObjekte.size(); i++)
        {
          SystemObject so = systemObjekte.get(i);
          
          boolean hatBerechtigung = _dav.hatBerechtigung(so, user.getPid(), f);
          
          if (hatBerechtigung)
          {
      
//            System.out.println(f.getFunktionsName() + " : " + true);
            
            if (!soFreigabe.contains(so))
              soFreigabe.add(so);
            if (!ofFreigabe.contains(f))
              ofFreigabe.add(f);
            flags[i] = true;
          }
          else
          {
//            System.out.println(f.getFunktionsName() + " : " + false);
            
            if (!soSperrung.contains(so))
              soSperrung.add(so);
            if (!ofSperrung.contains(f))
              ofSperrung.add(f);
  
            flags[i] = false;
          }
//          System.out.println(so.getPid() + " : " + f.getFunktionsName() + " : " + hatBerechtigung);
        }
  
        for (int i = 0; i < flags.length; i++)
        {
          if (!flags[i])
          {
            flag = false;
            // break;
          }
  
        }
  
      }
  
      for (SystemObject so : soSperrung)
      {
        if (soFreigabe.contains(so))
        {
          soFreigabe.remove(so);
        }
      }
  
      for (OberflaechenFunktion of : ofSperrung)
      {
        if (ofFreigabe.contains(of))
        {
          ofFreigabe.remove(of);
        }
      }
      
    }

    OberflaechenBerechtigungEreignis ereignis = new OberflaechenBerechtigungEreignis(this, flag, soFreigabe,
        soSperrung, ofFreigabe, ofSperrung);

    return ereignis;
  }

  public List<OberflaechenFunktion> getFunktionen()
  {
    return _funktionen;
  }

  public List<SystemObject> getSystemObjekte()
  {
    return _systemObjekte;
  }
  
  public void setSystemObjekte(List<SystemObject> systemObjekte)
  {
    this._systemObjekte = systemObjekte;
  }

  public static DaVKommunikation getDaV()
  {
    return _dav;
  }

  public static void setDaV(DaVKommunikation dav)
  {
    _dav = dav;
  }

}
