/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kalender;

import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Datenstrukturklasse, enthlt jeweils eine Zeitgruppe mit Start- und
 * Endzeitpunkt.
 * 
 * @author HofmannJ
 * 
 */
public class KalenderZeitFilterDatenStrukturZeitGruppe implements
		Comparable<KalenderZeitFilterDatenStrukturZeitGruppe>
{
	/**
	 * Startzeitpunkt als long
	 */
	private long _startzeitpunkt;

	/**
	 * Endzeitpunkt als long
	 */
	private long _endzeitpunkt;

	/**
	 * Konstruktor setzt die Parametern fr Start- und Endzeitpunkt
	 * 
	 * @param startzeitpunkt
	 *          Startzeit als long (Zeit in MS seit 1.1.1970)
	 * @param endzeitpunkt
	 *          Endzeit als long (Zeit in MS seit 1.1.1970)
	 */
	public KalenderZeitFilterDatenStrukturZeitGruppe(long startzeitpunkt,
			long endzeitpunkt)
	{
		_startzeitpunkt = startzeitpunkt;
		_endzeitpunkt = endzeitpunkt;
	}

	/**
	 * Konstruktor, setzt Start- und Endzeitpunkt.
	 * 
	 * @param startzeitpunkt
	 *          Startzeitpunkt als String. Dieser String muss dem Format
	 *          <code>HH:mm:s.S</code> gengen.
	 * @param endzeitpunkt
	 *          Endzeitpunkt als String. Dieser String muss dem Format
	 *          <code>HH:mm:s.S</code> gengen.
	 * @throws ParseException
	 *           Wird geworfen wenn der bergebene String von Start- oder
	 *           Endzeitpunkt nicht dem Format <code>HH:mm:s.S</code> gengt.
	 */
	public KalenderZeitFilterDatenStrukturZeitGruppe(String startzeitpunkt,
			String endzeitpunkt) throws ParseException
	{
		setStartzeitpunkt(startzeitpunkt);
		setEndzeitpunkt(endzeitpunkt);
	}

	/**
	 * Liefert den Endzeitpunkt
	 * 
	 * @return Endzeitpunkt (Zeit in MS seit 1.1.1970)
	 */
	public long getEndzeitpunkt()
	{
		return _endzeitpunkt;
	}

	/**
	 * Liefert den Endzeitpunkt als String.
	 * 
	 * @return Endzeitpunkt als String im Format <code>HH:mm:s.S</code>
	 */
	public String getEndzeitpunktAlsString()
	{
		String re = "";
		DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		re = df.format(new Date(_endzeitpunkt));
		return re;
	}

	/**
	 * Setzt den Endzeitpunkt
	 * 
	 * @param endzeitpunkt
	 *          Endzeitpunkt (Zeit in MS seit 1.1.1970)
	 */
	public void setEndzeitpunkt(long endzeitpunkt)
	{
		this._endzeitpunkt = endzeitpunkt;
	}

	/**
	 * Setzt den Endzeitpunkt.
	 * 
	 * @param endzeitpunkt
	 *          Endzeitpunkt als String. Dieser String muss dem Format
	 *          <code>HH:mm:s.S</code> gengen.
	 * @throws ParseException
	 *           Wird geworfen wenn der bergebene String nicht dem Format
	 *           <code>HH:mm:s.S</code> gengt.
	 */
	public void setEndzeitpunkt(String endzeitpunkt) throws ParseException
	{
		DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		Date endzeitDate = df.parse(endzeitpunkt);
		this._endzeitpunkt = endzeitDate.getTime();
	}

	/**
	 * Liefert den Startzeitpunkt
	 * 
	 * @return Startzeitpunkt (Zeit in MS seit 1.1.1970)
	 */
	public long getStartzeitpunkt()
	{
		return _startzeitpunkt;
	}

	public String getStartzeitpunktAlsString()
	{
		String re = "";
		DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		re = df.format(new Date(_startzeitpunkt));
		return re;
	}

	/**
	 * Setzt den Startzeitpunkt
	 * 
	 * @param startzeitpunkt
	 *          Startzeitpunkt (Zeit in MS seit 1.1.1970)
	 */
	public void setStartzeitpunkt(long startzeitpunkt)
	{
		this._startzeitpunkt = startzeitpunkt;
	}

	/**
	 * Setzt den Startzeitpunkt.
	 * 
	 * @param startzeitpunkt
	 *          Startzeitpunkt als String. Dieser String muss dem Format
	 *          <code>HH:mm:s.S</code> gengen.
	 * @throws ParseException
	 *           Wird geworfen wenn der bergebene String nicht dem Format
	 *           <code>HH:mm:s.S</code> gengt.
	 */
	public void setStartzeitpunkt(String startzeitpunkt) throws ParseException
	{
		DateFormat df = new SimpleDateFormat("HH:mm:s.S");
		Date startZeitDate = df.parse(startzeitpunkt);
		this._startzeitpunkt = startZeitDate.getTime();
	}

	@Override
	public int compareTo(KalenderZeitFilterDatenStrukturZeitGruppe o)
	{
		int re = 0;

		if (this._startzeitpunkt == o.getStartzeitpunkt())
		{
			if (_endzeitpunkt == o.getEndzeitpunkt())
			{
				re = 0;
			} else if (_endzeitpunkt < o.getEndzeitpunkt())
			{
				re = -1;
			} else if (_endzeitpunkt > o.getEndzeitpunkt())
			{
				re = 1;
			}
		} else if (this._startzeitpunkt < o.getStartzeitpunkt())
		{
			re = -1;
		} else if (this._startzeitpunkt > o.getStartzeitpunkt())
		{
			re = 1;
		}

		return re;
	}

	public String toString()
	{
		String re = "";



		re = re + "Zeitgruppe von: " + _startzeitpunkt + " ("
				+ getStartzeitpunktAlsString() + ") bis: " + _endzeitpunkt + " ("
				+ getEndzeitpunktAlsString() + ")";

		return re;
	}
}