/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.kontextmenu;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;

/**
 * Diese Klasse stellt den Generator fr ein Kontextmen zur Verfgung.
 * 
 * Nach einem Separator werden diese Eintrge, erweitert um eine Checkbox, dargestellt. Die selektierte Checkbox
 * selektiert die Action des Kontextmens, deren run-Methode fr einen Aufruf durch einen Doppelklick abfragbar ist.
 * 
 * @author HofmannJ
 * 
 */
public class KontextMenu
{
  /**
   * Speichert die {@link Action}s, welche im Kontextmen dargestellt werden sollen.
   */
  private LinkedList<Action> _eintraege = new LinkedList<Action>();

  /**
   * Referenziert die selektierte {@link Action}, deren run()-Methode bei einem Doppelklick aufgerufen werden soll.
   */
  private Action _selektierterEintrag = null;

  /**
   * Konstruktor
   */
  public KontextMenu()
  {
  }

  /**
   * Fgt eine {@link Action} zur Liste der {@link Action}s des Kontextmens hinzu. Es findet eine Prfung statt, ob
   * diese {@link Action} bereits im Kontextmen vorhanden ist. Ist dies der Fall, wird diese nicht erneut zum
   * Kontextmen hinzugefgt.
   * 
   * @param aktion
   *          Hinzuzufgende {@link Action}.
   */
  public void hinzufuegenAktion(Action aktion)
  {
    if (!_eintraege.contains(aktion))
    {
      _eintraege.add(aktion);
    }

  }

  /**
   * Entfernt eine {@link Action}, sofern in der Liste der {@link Action}s vorhanden, aus der Liste.
   * 
   * @param aktion
   *          Zu entfernende {@link Action}.
   */
  public void entferneAktion(Action aktion)
  {
    if (_eintraege.contains(aktion))
    {
      _eintraege.remove(aktion);
    }

    if (_selektierterEintrag.equals(aktion))
    {
      _selektierterEintrag = null;
    }
  }

  /**
   * Erzeugt einen {@link MenuManager} fr das Kontextmen.<br>
   * <br>
   * Eclipse bedingt <b>muss</b> der zurckgelieferte {@link MenuManager} mittels <code>registerContextMenu</code>
   * bei der Seite der View oder dem Editor fr das jeweilige {@link Control}, z.B. einen {@link TableViewer},
   * registriert werden:<br>
   * <ul>
   * <li>View: <code>getSite().registerContextMenu(...)</code></li>
   * <li>Editor: <code>getEditorSite().registerContextMenu(...)</code></li>
   * </ul>
   * 
   * @param control
   *          {@link Control}, an dem das KontextMen verankert werden soll.
   * @return Liefert den {@link MenuManager} des Kontextmens, der bei der View oder dem Editor fr das jeweilige
   *         {@link Control} registriert werden muss.
   */
  public MenuManager getKontextMenu(Control control)
  {
    MenuManager menuMgr = new MenuManager("#PopupMenu");
    menuMgr.setRemoveAllWhenShown(true);
    menuMgr.addMenuListener(new IMenuListener() {
      public void menuAboutToShow(IMenuManager manager)
      {
        fuelleKontextMenu(manager);
      }
    });

    Menu menu = menuMgr.createContextMenu(control);
    control.setMenu(menu);

    return menuMgr;
  }

  /**
   * Diese Methode fllt das Kontextmen mit den registrierten {@link Action}s.
   * 
   * @param manager
   *          Der {@link IMenuManager}, dessen Kontextmen gefllt werden soll.
   */
  private void fuelleKontextMenu(IMenuManager manager)
  {
    LinkedList<SpecAction> sps = new LinkedList<SpecAction>();

    if (_eintraege.size() > 0)
    {
      Iterator<Action> it = _eintraege.iterator();
      while (it.hasNext())
      {
        Action action = it.next();
        manager.add(action);

        SpecAction sp = new SpecAction(action.getText(), action);
        sps.add(sp);
      }

      if (sps.size() > 0)
      {
        manager.add(new Separator());
        
        MenuManager doppelKlickMenuManager = new MenuManager("Doppelklick Aktion");        

        Iterator<SpecAction> itSp = sps.iterator();
        while (itSp.hasNext())
        {
          SpecAction specAction = itSp.next();
          doppelKlickMenuManager.add(specAction);
        }
        
        manager.add(doppelKlickMenuManager);
      }
    }
  }

  /**
   * Liefert die selektierte {@link Action} zurck.
   * 
   * @return Die selektierte {@link Action} oder <code>null</code>, wenn keine {@link Action} selektiert wurde.
   */
  public Action getSelektierteAktion()
  {
    return _selektierterEintrag;
  }

  /**
   * Fhrt die <code>run</code>-Methode der selektierten {@link Action} aus.
   * 
   * @return Liefert <code>true</code>, wenn die <code>run</code>-Methode ausgefhrt werden konnte, sonst
   *         <code>false</code> (z.B. wenn keine {@link Action} selektiert wurde!).
   */
  public boolean selektierteAktionAusfuehren()
  {
    boolean erfolgreich = false;

    if (_selektierterEintrag != null)
    {
      _selektierterEintrag.run();
      erfolgreich = true;
    }

    return erfolgreich;
  }

  /**
   * Diese Klasse liefert die selektierbaren {@link Action}s dieses speziellen Kontextmens.
   * 
   * @author HofmannJ
   * 
   */
  public class SpecAction extends Action
  {
    /**
     * Gemerkte {@link Action}, deren run-Methode ausgefhrt wird, wenn die {@link SpecAction} selektiert wird.
     */
    private Action _merkAktion = null;

    /**
     * Konstruktor
     * 
     * @param text
     *          Text der {@link SpecAction}.
     * @param merkAktion
     *          Zu merkende {@link Action}.
     */
    public SpecAction(String text, Action merkAktion)
    {
      super(text, SpecAction.AS_CHECK_BOX);
      this._merkAktion = merkAktion;

      if (merkAktion != null)
      {
        this.setImageDescriptor(merkAktion.getImageDescriptor());
      }

      if (_merkAktion.equals(_selektierterEintrag))
      {
        this.setChecked(true);
      }
      else
      {
        this.setChecked(false);
      }
    }

    /**
     * Setzt die zu merkende {@link Action}.
     * 
     * @param merkAktion
     *          Zu merkende {@link Action}.
     */
    public void setMerkAktion(Action merkAktion)
    {
      this._merkAktion = merkAktion;
    }

    /**
     * Liefert die gemerkte {@link Action}.
     * 
     * @return Gemerkte {@link Action} oder <code>null</code>, wenn keine {@link Action} gespeichert wurde.
     */
    public Action getMerkAction()
    {
      return _merkAktion;
    }

    /**
     * In dieser <code>run</code>-Methode wird die Selektion und Deselektion der Eintrgt im Men durchgefhrt.
     */
    public void run()
    {
      if (_selektierterEintrag != _merkAktion)
      {
        _selektierterEintrag = _merkAktion;
      }
      else
      {
        _selektierterEintrag = null;
      }
    }

  }

}
