/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.AttributeListDefinition;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Hilfsklasse, realisiert einen Baum zur Aufnahme der Zellenstruktur der
 * Kopfzeilen. Jeder {@link Knoten} verfgt ber nur einen Vorgnger und
 * beliebig viele Nachfolger.
 * 
 * @author HofmannJ
 * 
 */
public class Knoten
{
	/**
	 * Der Debugger
	 */
	private Debug _debug = Debug.getLogger();

	/**
	 * Speichert den Vorgnger Knoten. Wenn null, dann ist auch die Ebene 0 und es
	 * gibt keinen Vorgnger.
	 */
	private Knoten _vorgaenger = null;

	/**
	 * Liste der Nachfolger dieses Knotens. Ist die Liste leer, so hat dieser
	 * Knoten keinen Nachfolgeknoten!
	 */
	private LinkedList<Knoten> _nachfolger = new LinkedList<Knoten>();

	/**
	 * Wert des Knotens
	 */
	private Object _wert = null;

	private int _von = 0;

	private int _bis = 0;

	/**
	 * Start X der Zelle (oben Links)
	 */
	private int _vonX = -1;

	/**
	 * Start Y der Zelle (oben Links)
	 */
	private int _vonY = -1;

	/**
	 * End X der Zelle (unten Rechts)
	 */
	private int _bisX = -1;

	/**
	 * End Y der Zelle (unten Rechts)
	 */
	private int _bisY = -1;

	/**
	 * Konstruktor. bernimmt den Wert des Knotens.
	 * 
	 * @param wert
	 *          Wert des Knotens.
	 */
	public Knoten(Object wert)
	{
		this._wert = wert;
	}

	/**
	 * Setzt den Vorgngerknoten.
	 * 
	 * @param vorgaenger
	 *          Vorgngerknoten.
	 */
	public void setVorgaenger(Knoten vorgaenger)
	{
		this._vorgaenger = vorgaenger;
	}

	/**
	 * Fgt einen Nachfolgerknoten zur Liste der Nachfolger hinzu. In dem
	 * Nachfolgeknoten wird dieser Knoten als Vorgnger gesetzt. <br>
	 * <br>
	 * <b>Achtung:</b> Jeder Knoten kann nur einen Vorgnger haben und jeder
	 * Knoten kann nur ein mal in der Liste der Nachfolger auftauchen. Die
	 * Prfung, ob ein Knoten in der Hierarchie schon einmal verwendet wurde,
	 * bleibt dem Entwickler vorbehalten.
	 * 
	 * @param nachfolgeKnoten
	 *          Hinzuzufgender Nachfolgeknoten.
	 */
	public void addNachfolger(Knoten nachfolgeKnoten)
	{
		if (!_nachfolger.contains(nachfolgeKnoten))
		{
			nachfolgeKnoten.setVorgaenger(this);
			_nachfolger.add(nachfolgeKnoten);
		}
	}

	/**
	 * Entfernt einen nachfolgenden Knoten von der Liste der Nachfolger. Der
	 * Vorgnger des nachfolgenden Knotens wird auf <code>null</code> gesetzt.
	 * 
	 * @param nachfolgeKnoten
	 *          Zu entfernender, nachfolgender Knoten.
	 */
	public void removeNachfolger(Knoten nachfolgeKnoten)
	{
		if (_nachfolger.contains(nachfolgeKnoten))
		{
			nachfolgeKnoten.setVorgaenger(null);
			_nachfolger.remove(nachfolgeKnoten);
		}
	}

	/**
	 * Prft, ob der Knoten Nachfolger hat.
	 * 
	 * @return Liefert <code>true</code>, wenn der Knoten Nachfolger hat, sonst
	 *         <code>false</code>.
	 */
	public boolean hatNachfolger()
	{
		if (_nachfolger.size() == 0)
		{
			return false;
		} else
		{
			return true;
		}
	}

	/**
	 * Liefert die Liste der Nachfolger dieses Knoten zurck.
	 * 
	 * @return Liste der Nachfolger dieses Knotens.
	 */
	public LinkedList<Knoten> getNachfolger()
	{
		return _nachfolger;
	}

	/**
	 * Setzt den Wert des Knotens.
	 * 
	 * @param wert
	 *          Zu setzender Wert.
	 */
	public void setWert(Object wert)
	{
		this._wert = wert;
	}

	/**
	 * Liefert den Wert des Knotens.
	 * 
	 * @return Wert des Knotens.
	 */
	public Object getWert()
	{
		return _wert;
	}

	/**
	 * Prft, ob der Knoten einen Vorgngerknoten hat.
	 * 
	 * @return Liefert <code>true</code>, wenn dieser Knoten einen
	 *         Vorgngerknoten hat, sonst <code>false</code>.
	 */
	public boolean hatVorgaenger()
	{
		if (_vorgaenger != null)
		{
			return true;
		} else
		{
			return false;
		}
	}

	/**
	 * Liefert den Elternknoten dieses Knotens
	 * 
	 * @return ElternKnoten
	 */
	public Knoten getParent()
	{
		return _vorgaenger;
	}

	/**
	 * Bestimmt rekursiv eine Liste der Vorgnger dieses Knotens. Der erste Knoten
	 * ohne Vorgnger befindet sich am Anfang dieser Liste, dieser aktuelle Knoten
	 * befindet sich am Ende der Liste.
	 * 
	 * @return Liste der Vorgnger mit dem aktuellen Knoten am Ende
	 */
	public LinkedList<Knoten> getVorgaenger()
	{
		LinkedList<Knoten> re = new LinkedList<Knoten>();
		re.add(this);
		if (hatVorgaenger())
		{
			re.addAll(0, _vorgaenger.getVorgaenger());
		}
		return re;
	}

	/**
	 * Liefert die Tiefe des Knotens zurck. Wird rekursiv berechnet.
	 * 
	 * @return Tiefe des Knotens.
	 */
	public int getTiefe()
	{
		int re = 0;

		if (hatVorgaenger())
		{
			re++;

			int tiefe = _vorgaenger.getTiefe();
			re = re + tiefe;
		}

		return re;
	}

	/**
	 * Liefert den Pfad, auf dem dieser {@link Knoten} erreichbar ist. Der erste
	 * {@link Knoten} in der Liste entspricht dem Wurzelknoten und verfgt ber
	 * keinen weiteren Vorgnger.
	 * 
	 * @return Pfad vom Wurzelknoten zu diesem {@link Knoten}.
	 */
	public LinkedList<Knoten> getPfad()
	{
		LinkedList<Knoten> pfad = new LinkedList<Knoten>();
		pfad.add(this);

		if (hatVorgaenger())
		{
			LinkedList<Knoten> vorgaengerPfad = _vorgaenger.getPfad();
			pfad.addAll(0, vorgaengerPfad);
		}

		return pfad;
	}

	/**
	 * Liefert den Wurzelknoten des Baumes.
	 * 
	 * @return Wurzelknoten des Baumes.
	 */
	public Knoten getWurzelKnoten()
	{
		return getPfad().get(0);
	}

	/**
	 * Bestimmt die Breite der breitesten Ebene.
	 * 
	 * @return Breite der breitesten Ebene.
	 */
	public int bestimmeBreitesteEbene()
	{
		int breitesteEbene = 0;
		Knoten wurzelKnoten = getWurzelKnoten();
		int tiefsteEbene = wurzelKnoten.bestimmeTiefsteEbene();

		for (int i = 0; i <= tiefsteEbene; i++)
		{
			LinkedList<Knoten> ebene = wurzelKnoten.bestimmeKnotenAufEbene(i);
			if (ebene.size() > breitesteEbene)
			{
				breitesteEbene = ebene.size();
			}

			Iterator<Knoten> it = ebene.iterator();
			while (it.hasNext())
			{
				Knoten knoten = it.next();
				_debug.fine("Knoten auf Ebene " + i + ": "
						+ knoten.getSpaltenBezeichnung());

			}
		}

		return breitesteEbene;
	}

	/**
	 * Bestimmt die Knoten, die sich auf der gleichen Ebene befinden.
	 * 
	 * @param ebene
	 *          Ebene
	 * @return Knoten, die auf der angegebenen ebene liegen.
	 */
	public LinkedList<Knoten> bestimmeKnotenAufEbene(int ebene)
	{
		LinkedList<Knoten> knotenListe = null;

		/*
		 * Wurzelknoten bestimmen
		 */
		Knoten wurzelKnoten = this.getWurzelKnoten();

		/*
		 * Knoten auf gleicher Ebene bestimmen.
		 */
		knotenListe = bestimmtKnotenAufEbeneRekursiv(wurzelKnoten, ebene);

		return knotenListe;
	}

	/**
	 * Bestimmt die Knoten, die sich auf der gleichen Ebene befinden.
	 * DFS-Traversierung.
	 * 
	 * @param knoten
	 *          Hierbei sollte es sich initial um den Wurzelknoten handeln.
	 * @param ebene
	 *          Ebene
	 * @return Liste der Knoten auf der angegebenen Ebene.
	 */
	private LinkedList<Knoten> bestimmtKnotenAufEbeneRekursiv(Knoten knoten,
			int ebene)
	{
		LinkedList<Knoten> knotenListe = new LinkedList<Knoten>();

		int tiefeKnoten = knoten.getTiefe();

		if (tiefeKnoten == ebene)
		{
			knotenListe.add(knoten);
		} else if (tiefeKnoten < ebene)
		{
			Iterator<Knoten> it = knoten.getNachfolger().iterator();
			while (it.hasNext())
			{
				Knoten nachfolgeKnoten = it.next();
				knotenListe.addAll(nachfolgeKnoten.bestimmtKnotenAufEbeneRekursiv(
						nachfolgeKnoten, ebene));
			}
		}

		return knotenListe;
	}

	/**
	 * Bestimmte die tiefste Ebene im Baum, ausgehend vom Wurzelknoten.
	 * 
	 * @return Liefert die tiefste Ebene im Baum vom Wurzelknoten ausgehend.
	 */
	public int bestimmeTiefsteEbene()
	{
		Knoten wurzelKnoten = this.getWurzelKnoten();
		return bestimmeTiefsteEbeneRekursiv(wurzelKnoten);
	}

	/**
	 * Bestimmt rekursiv die tiefste Ebene des Baumes.
	 * 
	 * @param knoten
	 *          Wurzeknoten, von dem ausgehend die tiefste Ebene bestimmt werden
	 *          soll.
	 * @return Liefert die tiefste Ebene innerhalb der jeweiligen Rekursionsstufe
	 *         zurck.
	 */
	public int bestimmeTiefsteEbeneRekursiv(Knoten knoten)
	{
		int tiefeKnoten = knoten.getTiefe();

		if (knoten.getNachfolger().size() > 0)
		{
			Iterator<Knoten> it = knoten.getNachfolger().iterator();
			while (it.hasNext())
			{
				Knoten nachfolgeKnoten = it.next();
				int tiefeNachfolger = nachfolgeKnoten
						.bestimmeTiefsteEbeneRekursiv(nachfolgeKnoten);
				if (tiefeNachfolger > tiefeKnoten)
				{
					tiefeKnoten = tiefeNachfolger;
				}
			}
		}

		return tiefeKnoten;
	}

	/**
	 * Bestimmt die Knoten auf der tiefsten Ebene, welche ber keinen Nachfolger
	 * verfgen.
	 * 
	 * @return Knoten auf der tiefsten Ebene ohne Nachfogler
	 */
	public LinkedList<Knoten> bestimmeKnotenOhneNachfolger()
	{
		Knoten wurzelKnoten = this.getWurzelKnoten();
		return bestimmeKnotenOhneNachfolgerRek(wurzelKnoten);
	}

	/**
	 * Bestimmt rekursiv die Knoten auf der tiefsten Ebene, welche ber keinen
	 * Nachfolger verfgen.
	 * 
	 * @param knoten
	 *          Ausgangsknoten, von dem aus der Algorithmus los rennt.
	 * @return Liste von Knoten auf der tiefsten Ebene.
	 */
	public LinkedList<Knoten> bestimmeKnotenOhneNachfolgerRek(Knoten knoten)
	{
		LinkedList<Knoten> re = new LinkedList<Knoten>();

		if (knoten.getNachfolger().size() == 0)
		{
			re.add(knoten);
		} else
		{
			Iterator<Knoten> it = knoten.getNachfolger().iterator();
			while (it.hasNext())
			{
				Knoten nachfolgeKnoten = it.next();
				re.addAll(nachfolgeKnoten
						.bestimmeKnotenOhneNachfolgerRek(nachfolgeKnoten));
			}
		}

		return re;
	}

	/**
	 * Liefert die Bezeichnung der Spalte.
	 * 
	 * @return Bezeichnung der Spalte
	 */
	public String getSpaltenBezeichnung()
	{
		String re = null;

		if (_wert != null)
		{
			if (_wert instanceof AttributeGroup)
			{
				AttributeGroup atgWert = (AttributeGroup) _wert;
				// System.out.println("====>" + atgWert.getPid() + "<====");
				re = atgWert.getName();
			} else if (_wert instanceof Attribute)
			{
				Attribute att = (Attribute) _wert;
				// System.out.println("====>" + att.getPid() + att.getName() + "<====");
				re = att.getName();

				/*
				 * Wenn es sich um ein Array handelt, dann [..] an die
				 * Spaltenbezeichnung anhngen.
				 */
				if (att.isArray())
				{
					re = re + " [..]";
				}

			} else if (_wert instanceof AttributeListDefinition)
		  {
				AttributeListDefinition att = (AttributeListDefinition) _wert;
				// System.out.println("====>" + att.getPid() + att.getName() + "<====");
				re = att.getName();

				re += " Liste ";
				
				/*
				 * Wenn es sich um ein Array handelt, dann [..] an die
				 * Spaltenbezeichnung anhngen.
				 */
//				if (att.isArray())
//				{
//					re = re + " [..]";
//				}
		  } else if (_wert instanceof Data)
			{
				re = ((Data) _wert).getName();
			} else
			{
				re = "Unbekannt";
			}
		}

		return re;
	}

	/**
	 * Prft, ob es sich bei diesem Knoten um ein Array handelt.
	 * 
	 * @return Liefert <code>true</code>, wenn es sich um ein Array handelt,
	 *         sonst <code>false</code>.
	 */
	protected boolean isArray()
	{
		boolean re = false;

		if (_wert != null)
		{
			if (_wert instanceof Attribute)
			{
				Attribute att = (Attribute) _wert;

				/*
				 * Wenn es sich um ein Array handelt, dann [..] an die
				 * Spaltenbezeichnung anhngen.
				 */
				if (att.isArray())
				{
					re = true;
				}

			}
		}

		return re;
	}

	public String toString()
	{
		String re = null;

		/*
		 * Attributgruppe
		 */
		if (_wert instanceof AttributeGroup)
		{
			AttributeGroup atgWert = (AttributeGroup) _wert;
			re = "Attributgruppe" + atgWert.getName();
		}
		/*
		 * Attribut
		 */
		else if (_wert instanceof Attribute)
		{
			Attribute att = (Attribute) _wert;

			/*
			 * Normales Attribut oder AttributeListDefinition
			 */
			if (att.getType() instanceof AttributeListDefinition)
			{
				re = "Attribut (ATL): " + att.getName();
			} else
			{
				re = "Attribut: " + att.getName();
			}
		} else
		{
			re = "Knoten (" + _wert.toString() + ")";
		}

		return re;
	}

	public int getVon()
	{
		return _von;
	}

	public void setVon(int von)
	{
		_von = von;
	}

	public int getBis()
	{
		return _bis;
	}

	public void setBis(int bis)
	{
		_bis = bis;
	}

	public boolean istInBereich(int wert)
	{

		if ((wert >= _von) && (wert <= _bis))
		{
			// System.out.println("WERT: " + wert + " VON: " + _von + " BIS: " +
			// _bis
			// + " = " + true);
			return true;
		} else
		{
			// System.out.println("WERT: " + wert + " VON: " + _von + " BIS: " +
			// _bis
			// + " = " + false);
			return false;
		}
	}

	public int getVonX()
	{
		return _vonX;
	}

	public void setVonX(int vonX)
	{
		this._vonX = vonX;
	}

	public int getVonY()
	{
		return _vonY;
	}

	public void setVonY(int vonY)
	{
		this._vonY = vonY;
	}

	public int getBisX()
	{
		return _bisX;
	}

	public void setBisX(int bisX)
	{
		this._bisX = bisX;
	}

	public int getBisY()
	{
		return _bisY;
	}

	public void setBisY(int bisY)
	{
		this._bisY = bisY;
	}

	/**
	 * Prft, ob sich eine Zelle im Bereich eines Knotens befindet.
	 * 
	 * @param zeile
	 *          Zeile der Zelle.
	 * @param spalte
	 *          Spalte der Zelle.
	 * @return Liefert <code>true</code>, wenn sich die Zelle im Bereich des
	 *         Knotens befindet, sonst <code>false</code>.
	 */
	public boolean istInBereich(int zeile, int spalte)
	{
		boolean re = false;

		if ((spalte >= _vonX) && (spalte <= _bisX))
		{
			if ((zeile >= _vonY) && (zeile <= _bisY))
			{
				re = true;
			}
		}
		return re;
	}

	public LinkedList<Knoten> getListeAllerKnotenDfs()
	{
		Knoten wurzelKnoten = getWurzelKnoten();
		LinkedList<Knoten> re = getListeAllerKnotenDfsRekursiv(wurzelKnoten);

		return re;
	}

	public LinkedList<Knoten> getListeAllerKnotenDfsRekursiv(Knoten knoten)
	{
		LinkedList<Knoten> re = new LinkedList<Knoten>();

		re.add(knoten);
		Iterator<Knoten> itNachfolger = knoten.getNachfolger().iterator();
		while (itNachfolger.hasNext())
		{
			Knoten nachfolgeKnoten = itNachfolger.next();
			re.addAll(getListeAllerKnotenDfsRekursiv(nachfolgeKnoten));
		}

		return re;
	}

}
