/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.listendarstellung;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.swt.graphics.Point;

import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.CellRenderer;
import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.FaltStatusSpalten;
import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.FixedCellRendererFaltbar;
import de.bsvrz.buv.rw.basislib.listendarstellung.ktableerweiterungen.IFaltStautsSpalten;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.kupzog.ktable.KTable;
import de.kupzog.ktable.KTableCellEditor;
import de.kupzog.ktable.KTableCellRenderer;
import de.kupzog.ktable.KTableDefaultModel;
import de.kupzog.ktable.editors.KTableCellEditorText;

/**
 * Erweitertes Tabellenmodell fr die Anzeige von SystemObjekten mit Ihren
 * Attributgruppen in einer {@link KTable}. Hierzu sind Attributgruppe und
 * Aspekt vorzugeben. Die Methode
 * {@link TabellenModell#addDTO(DataTableObjectErweitert)} ermglicht es,
 * weitere {@link DataTableObjectErweitert} zur Liste der TabellenObjekte
 * hinzuzufgen. Diese beinhalten die notwendigen Daten und knnen auch im
 * nachhinein zum Modell hinzugefgt werden.
 * 
 * @author HofmannJ
 * 
 */
public class TabellenModell extends KTableDefaultModel implements
		IFaltStautsSpalten
{

	/**
	 * Debug-Flag
	 */
	private boolean _debug = false;

	public static final int STYLE_FLAT = 1 << 2;
	private FixedCellRendererFaltbar _fixedCellRendererDambach = new FixedCellRendererFaltbar(
			STYLE_FLAT);

	/**
	 * Speichert die Faltungen der Spalten
	 */
	private FaltStatusSpalten _faltStatusSpalten = new FaltStatusSpalten();

	private ListenDarstellungBestimmeKopfzeilen _kopfzeilen = null;

	/**
	 * Wurzelknoten der Kopfzeile
	 */
	private Knoten _wurzelKopfZeile = null;

	/**
	 * Tiefste Ebene der Kopfzeile
	 */
	private int _tiefsteEbeneKopfzeile = 0;

	/**
	 * Offset, wann die Spalten beginnen. Auf der linken Seite werden die Objekte
	 * mit Datum dargestellt (Spalte 0, 1 und 2).
	 */
	private int _spaltenOffset = 3;

	/**
	 * Verbindung zum DaV. Wird fr sptere Zwecke vorgehalten. Bei weiterer
	 * Verwendung ist die SuppressWarnings Annotation zu entfernen!
	 */
	@SuppressWarnings("unused")
	private ClientDavInterface _davVerbindung = null;

	/**
	 * Lister aller Knoten, zusammengestellt via Tiefensuche.
	 */
	private LinkedList<Knoten> _listeAllerKnotenDfs = null;

	/**
	 * Liste aller Objekte, die in der Tabelle dargestellt werden sollen
	 */
	private LinkedList<DataTableObjectErweitert> _tabellenObjekte = new LinkedList<DataTableObjectErweitert>();

	private int _fixedHeaderRowCount = 0;

	/**
	 * Anzahl der Zeilen im Datenbereich
	 */
	private int _rowCount = 0;

	/**
	 * Der Renderer fr die Zellen
	 */
	private CellRenderer _cellRenderer = null;

	/**
	 * Definiert die Art der Darstellung. Die Voreinstellung ist
	 * {@link EnumDarstellungsArt#zustandsDarstellung}.
	 */
	private EnumDarstellungsArt _darstellungsArt = EnumDarstellungsArt.zustandsDarstellung;

	/**
	 * Konstruktor
	 * 
	 * @param dav
	 *          Verbindung zum Datenverteiler
	 * @param typ
	 *          Typ der Objekte.
	 * @param atg
	 *          Attributgruppe
	 * @param asp
	 *          Aspekt
	 * @param tabellenObjekte
	 *          Liste von Tabellenobjekten, die zur Darstellung gebracht werden
	 *          sollen. Ihr Typ, Attributruppe und Aspekt mssen den Angaben der
	 *          drei vorhergehenden Parameter entsprechen. Kann auch
	 *          <code>null</code> sein, dann wird eine leere Liste erzeugt, welche
	 *          nachtrglich durch
	 *          {@link TabellenModell#addDTO(DataTableObjectErweitert)} befllbar
	 *          ist.
	 */
	public TabellenModell(ClientDavInterface dav, String typ, String atg,
			String asp, LinkedList<DataTableObjectErweitert> tabellenObjekte)
	{

		super();

		this._davVerbindung = dav;

		if (tabellenObjekte != null)
		{
			this._tabellenObjekte = tabellenObjekte;
		}

		/*
		 * Initialisierung des TabellenKopfes
		 */
		initialisiereKopf(dav, typ, atg, asp);

		/*
		 * Initialisierung des Datenbereiches
		 */
		initialisiereDaten();

		// // Test: Ausgabe aller Tabellenobjekte mit Start und Endzeile:
		// Iterator<DataTableObjectErweitert> it = _tabellenObjekte.iterator();
		// int i = 0;
		// while (it.hasNext()) {
		// DataTableObjectErweitert tbo = it.next();
		// int startZeile = tbo.getStartOffsetZeile();
		// int endZeile = tbo.getEndOffsetZeile();
		// System.out.println("=>DataTableObjectErweitert #" + i + "
		// StartZeile: "
		// + startZeile + " EndZeile: " + endZeile);
		// i++;
		// }

	}

	/**
	 * Setzt die Darstellungsart.
	 * 
	 * @param darstellungsArt
	 *          {@link EnumDarstellungsArt}
	 */
	public void setDarstellungsArt(EnumDarstellungsArt darstellungsArt)
	{
		this._darstellungsArt = darstellungsArt;
	}

	/**
	 * Liefert die Darstellungsart.
	 * 
	 * @return {@link EnumDarstellungsArt}
	 */
	public EnumDarstellungsArt getDarstellungsArt()
	{
		return _darstellungsArt;
	}

	/**
	 * Initialisiert den Kopf der Tabelle
	 * 
	 * @param dav
	 *          Verbindung zum Datenverteiler
	 * @param typ
	 *          Objekt Typ
	 * @param atg
	 *          Attributgruppe
	 * @param asp
	 *          Aspekt
	 */
	private void initialisiereKopf(ClientDavInterface dav, String typ,
			String atg, String asp)
	{

		_kopfzeilen = new ListenDarstellungBestimmeKopfzeilen(dav);
		_kopfzeilen.setDaten(typ, atg, asp);
		_kopfzeilen.bestimmeAttributeEinerAttributGruppe();
		_wurzelKopfZeile = _kopfzeilen.getWurzelKnoten();
		_tiefsteEbeneKopfzeile = _wurzelKopfZeile.bestimmeTiefsteEbene();
		berechneBreitenKopf();
		berecheKombiBereicheKopf();
	}

	/**
	 * Berechtnet die Breiten aller einzeiligen Zellen
	 */
	private void berechneBreitenKopf()
	{
		for (int i = 0; i <= _tiefsteEbeneKopfzeile; i++)
		{
			// _debug.fine("Berechne Breiten der Ebene: " + i);

			LinkedList<Knoten> erwKnoten = new LinkedList<Knoten>();
			Knoten vorgaenger = null;

			Iterator<Knoten> it = _wurzelKopfZeile.bestimmeKnotenAufEbene(i)
					.iterator();
			while (it.hasNext())
			{
				Knoten knoten = it.next();

				int breite = knoten.bestimmeKnotenOhneNachfolgerRek(knoten).size();

				// int tiefe = knoten.getTiefe();
				// System.out.println("Knoten:" + knoten.getSpaltenBezeichnung()
				// + " Tiefe: " + tiefe + " Breite: " + breite);

				if (vorgaenger == null)
				{
					int von = 0;
					int bis = von + breite - 1;

					knoten.setVon(von);
					knoten.setBis(bis);
					erwKnoten.add(knoten);

					vorgaenger = knoten;

				} else
				{

					int von = vorgaenger.getBis() + 1;
					int bis = von + breite - 1;
					knoten.setVon(von);
					knoten.setBis(bis);

					erwKnoten.add(knoten);
					vorgaenger = knoten;
				}
			}
		}
	}

	/**
	 * Berechnet die Bereiche (vonX, vonY, bisX, bisY) aller Zellen.
	 */
	private void berecheKombiBereicheKopf()
	{
		for (int ebene = 0; ebene <= _tiefsteEbeneKopfzeile; ebene++)
		{
			LinkedList<Knoten> knotenAufEbene = _wurzelKopfZeile
					.bestimmeKnotenAufEbene(ebene);
			Iterator<Knoten> it = knotenAufEbene.iterator();
			while (it.hasNext())
			{
				Knoten knoten = it.next();
				if (!knoten.hatNachfolger())
				{
					knoten.setVonX(knoten.getVon());
					knoten.setVonY(ebene);
					knoten.setBisX(knoten.getBis());
					knoten.setBisY(_tiefsteEbeneKopfzeile);

					// System.out.println("Knoten vonX: " + knoten.getVonX() + "
					// vonY: "
					// + knoten.getVonY() + " bisX:" + knoten.getBisX() + "
					// bisY: "
					// + knoten.getBisY());

				} else
				{
					// System.out.println("Knoten " +
					// knoten.getSpaltenBezeichnung()
					// + " hat Nachfolger");

					knoten.setVonX(knoten.getVon());
					knoten.setVonY(ebene);
					knoten.setBisX(knoten.getBis());
					knoten.setBisY(ebene);

					// System.out.println("Knoten vonX: " + knoten.getVonX() + "
					// vonY: "
					// + knoten.getVonY() + " bisX:" + knoten.getBisX() + "
					// bisY: "
					// + knoten.getBisY());

				}
			}
		}

		/**
		 * Bestimmung aller Knoten via DFS
		 */
		_listeAllerKnotenDfs = _wurzelKopfZeile.getListeAllerKnotenDfs();

	}

	/**
	 * Fhrt eine komplette Initialisierung der Daten dieser Tabelle durch. Diese
	 * Methode sollte initial aufgerufen werden, wenn eine Liste von
	 * {@link DataTableObjectErweitert} zur Anzeige bergeben wird.<br>
	 * Wird mittels der Methode
	 * {@link TabellenModell#addDTO(DataTableObjectErweitert)} nachtrglich ein
	 * {@link DataTableObjectErweitert} hinzugefgt, ist diese Methode
	 * <b>nicht</b> aufzurufen.
	 */
	protected void initialisiereDaten()
	{
		/*
		 * Datenbereich, Auswertung der Zellen
		 */

		int startOffsetZeile = 0;

		if ((_tabellenObjekte != null) && (_tabellenObjekte.size() > 0))
		{
			Iterator<DataTableObjectErweitert> it = _tabellenObjekte.iterator();
			while (it.hasNext())
			{
				DataTableObjectErweitert dataTableObjectErweitert = it.next();

				startOffsetZeile = initialisiereEinzelnesDTO(dataTableObjectErweitert,
						startOffsetZeile);
			}

		}

		if (_wurzelKopfZeile != null)
		{
			_fixedHeaderRowCount = _wurzelKopfZeile.bestimmeTiefsteEbene() + 1;
		}

		/*
		 * Initialisierung nach hinzufgen eines DataTableObjecterweitert
		 */
		initialisiereNachHinzufuegenDTO();
	}

	/**
	 * Fhrt abschlieende Initialisierungsoperationen nach dem Hinzufgen eines
	 * oder mehrerer {@link DataTableObjectErweitert} durch. Dazu gehren:<br>
	 * <ul>
	 * <li>Bestimmung der Anzahl der Zeilen (rowCount)</li>
	 * <li>Bestimmung von geraden und ungeraden Zeilen zwecks Frbung</li>
	 * </ul>
	 */
	private void initialisiereNachHinzufuegenDTO()
	{
		/*
		 * Berechnung der Anzahl der Zeilen, damit dies nicht stets aufs neue von
		 * doGetRowCount durchgefhrt werden muss.
		 */
		_rowCount = berechneAnzahlZeilen();

		/*
		 * Bestimmen von geraden und ungeraden DataTableObjectErweitert zwecks
		 * Zeileneinfrbung.
		 */
		bestimmeGeradeUndUngeradeDataTableObjectErweitertZeilen();
	}

	/**
	 * Initialisiert ein einzelnes {@link DataTableObjectErweitert}
	 * 
	 * @param dataTableObjectErweitert
	 *          Zu initialisierendes {@link DataTableObjectErweitert}.
	 * @param startOffsetZeile
	 *          Startoffset der Zeilen des zu initialisierenden
	 *          {@link DataTableObjectErweitert}.
	 * @return Liefert das StartOffset der Zeilen fr ein nchstes
	 *         {@link DataTableObjectErweitert}. Dies kann natrlich nur dann
	 *         verwendet werden, wenn dieses {@link DataTableObjectErweitert} in
	 *         der Liste der TabellenObjekte aufgenommen wurde.
	 */
	private int initialisiereEinzelnesDTO(
			DataTableObjectErweitert dataTableObjectErweitert, int startOffsetZeile)
	{

		/*
		 * Setzen des Start-Offsets
		 */
		dataTableObjectErweitert.setStartOffsetZeile(startOffsetZeile);
		// System.out.println("\t==>StartOffsetZeile: " +
		// startOffsetZeile);

		/*
		 * Besorgen des Data zwecks Auswertung der Zellenstruktur
		 */
		Data data = dataTableObjectErweitert.getData();

		/*
		 * Erzeugen & Setzen der Wurzelzelle
		 */
		Zelle wurzelZelle = new Zelle(data);
		dataTableObjectErweitert.setWurzelZelle(wurzelZelle);

		if (data != null)
		{
			if (data.isList())
			{
				wurzelZelle.setListe(true);
			} else if (data.isArray())
			{
				wurzelZelle.setArray(true);
			} else if (data.isPlain())
			{
				wurzelZelle.setPlain(true);
			}
		} else
		{
			wurzelZelle.setPlain(true);
			wurzelZelle.setWert(dataTableObjectErweitert.getText());
			wurzelZelle.setBreite(_wurzelKopfZeile.bestimmeKnotenOhneNachfolger()
					.size());
		}

		/*
		 * Starten der Auswertung. Nachfolgende Kindzellen werden bercksichtigt.
		 */

		auswertenZelle(wurzelZelle);

		// System.err.println("Wurzlbreite: " + wurzelZelle.getBreite(wurzelZelle));

		/*
		 * Bestimmen aller Plain-Zellen
		 */
		dataTableObjectErweitert.setPlainZellen(wurzelZelle
				.bestimmeAllePlainZellen());

		Iterator<Zelle> itPlainZellen = dataTableObjectErweitert.getPlainZellen()
				.iterator();

		while (itPlainZellen.hasNext())
		{
			Zelle zelle = itPlainZellen.next();

			zelle.setStartX(zelle.bestimmeStartPosX(zelle));
			zelle.setStartY(zelle.bestimmeStartPosY(zelle));
			zelle.setEndX(zelle.bestimmeEndPosX(zelle));

			// System.out.println("ZELLE(" + zelle.getDarstellung() + "): SX: "
			// + zelle.getStartX() + " SY: " + zelle.getStartY() + " EX: "
			// + zelle.getEndX());

		}

		/*
		 * Bestimmen der EndPosY
		 */
		dataTableObjectErweitert.getWurzelZelle().bestimmeEndPosY(
				dataTableObjectErweitert.getWurzelZelle()); // Eingefgt

		/*
		 * Setzen der tiefsten Ebene. Damit ist auch das Endoffset der Zeilen
		 * bestimmt
		 */
		int tiefsteEbene = wurzelZelle
				.bestimmeTiefstesElementImTiefstenArrayInNachfolgenderZelle(wurzelZelle);
		// System.out.println("Tiefer: " + tiefsteEbene);
		dataTableObjectErweitert.setTiefsteEbene(tiefsteEbene);

		if (_debug)
		{
			if (data == null)
			{
				int start = dataTableObjectErweitert.getStartOffsetZeile();
				int ende = dataTableObjectErweitert.getEndOffsetZeile();

				System.out.println("S: " + start + " E: " + ende
						+ " PlainZellen Anzahl: "
						+ dataTableObjectErweitert.getPlainZellen().size());
			}
		}

		/*
		 * Erhhen des StartOffsets der Zeilen
		 */
		int naechsteStartOffsetZeile = dataTableObjectErweitert.getEndOffsetZeile() + 1;

		// System.out.println("DATA: "
		// + dataTableObjectErweitert.getTiefsteEbene());
		// System.out.println("<======");

		// dataTableObjectErweitert.initialisiereMap();

		/*
		 * Ausgabe Plain-Zellen
		 */
		// itPlainZellen = dataTableObjectErweitert.getPlainZellen().iterator();
		// while (itPlainZellen.hasNext())
		// {
		// Zelle zelle = itPlainZellen.next();
		//
		// System.out.println("ZELLE(" + zelle.getDarstellung() + "): SX: "
		// + zelle.getStartX() + " SY: " + zelle.getStartY() + " EX: "
		// + zelle.getEndX() + " EY: " + zelle.getEndY());
		//
		// }
		return naechsteStartOffsetZeile;
	}

	/*
	 * private void analysiere(Data data, String rand) { Iterator it =
	 * data.iterator();
	 * 
	 * while (it.hasNext()) { Data data1 = (Data) it.next();
	 * 
	 * if (data1.isArray()) { analysiere(data1, rand + " "); }
	 * 
	 * if (data1.isList()) { analysiere(data1, rand + " "); }
	 * 
	 * if (data1.isArray()) { System.out.println("Achtung Array"); }
	 * 
	 * System.out.println(rand + "ANALYSIERE " + data1.toString()); } }
	 */

	/**
	 * Luft ber die Liste der TabellenObjekte vom Typ
	 * {@link DataTableObjectErweitert} und prft, ob die aktuelle Listenposition
	 * gerade oder ungerade ist. Im jeweiligen {@link DataTableObjectErweitert}
	 * wird das entsprechende Flag gesetzt. Auf diese Weise lassen sich die Zellen
	 * bzw. Zeilen von geraden/ungeraden {@link DataTableObjectErweitert}
	 * unterschiedlich einfrben.
	 */
	private void bestimmeGeradeUndUngeradeDataTableObjectErweitertZeilen()
	{
		int i = 0;
		Iterator<DataTableObjectErweitert> it = _tabellenObjekte.iterator();
		while (it.hasNext())
		{
			DataTableObjectErweitert objectErweitert = it.next();
			if (i % 2 == 0)
			{
				objectErweitert.setGeradeZeile(true);
			} else
			{
				objectErweitert.setGeradeZeile(false);
			}

			i++;
		}
	}

	/**
	 * Hinzufgen eines {@link DataTableObjectErweitert} zur aktuellen
	 * Darstellung. Die Daten werden an die Liste der TabellenObjekte angehngt
	 * und es wird eine Initialisierung fr das entsprechende
	 * {@link DataTableObjectErweitert} durchgefhrt.<br>
	 * <br>
	 * <b>Achtung:</b> Auf das {@link KTable}-Widget ist hiernach ein redraw
	 * aufzurufen, das sonst keine Aktualisierung der Darstellung der Tabelle
	 * erfolgt.
	 * 
	 * @param wert
	 *          {@link DataTableObjectErweitert}, welches zur Liste der der
	 *          TabellenObjekte hinzugefgt wird.
	 */
	public void addDTO(DataTableObjectErweitert wert)
	{

//		System.out.println("addDTO => " + wert.getData());

		if (_tabellenObjekte.size() == 0)
		{
			wert.setAenderungsDarstellung(true);
		} else
		{
			DataTableObjectErweitert letztesAenderungsDarstellungsObjekt = bestimmeLetztesAenderungsDarstellungsObjekt();

			String dataAlt = "";
			if (letztesAenderungsDarstellungsObjekt.getData() != null)
			{
				dataAlt = letztesAenderungsDarstellungsObjekt.getData().toString();
			} else
			{
				dataAlt = letztesAenderungsDarstellungsObjekt.getText();
			}

			String dataNeu = "";
			if (wert.getData() != null)
			{
				dataNeu = wert.getData().toString();
			} else
			{
				dataNeu = wert.getText();
			}

			if (!dataNeu.equals(dataAlt))
			{
//				System.out.println("===>TRUE");
				wert.setAenderungsDarstellung(true);
			}
//			else
//			{
//				System.out.println("===>FALSE");
//			}

		}

		@SuppressWarnings("unused")
		int startOffset = 0;
		if (_tabellenObjekte.size() == 0)
		{
			startOffset = 0;
		} else if (_tabellenObjekte.size() > 0)
		{
			if ((_darstellungsArt.equals(EnumDarstellungsArt.aenderungsDarstellung))
					&& (wert.isAenderungsDarstellung()))
			{
				startOffset = bestimmeLetztesAenderungsDarstellungsObjekt()
						.getEndOffsetZeile() + 1;
			} else
			{
				if (!_darstellungsArt.equals(EnumDarstellungsArt.aenderungsDarstellung))
					startOffset = (_tabellenObjekte.get((_tabellenObjekte.size() - 1)))
							.getEndOffsetZeile() + 1;
			}
		}

		if ((_darstellungsArt.equals(EnumDarstellungsArt.zustandsDarstellung))
				|| ((_darstellungsArt.equals(EnumDarstellungsArt.aenderungsDarstellung)) && (wert
						.isAenderungsDarstellung())))
		{
			_tabellenObjekte.add(wert);
//			System.err.println("=> initialisiereEinzelnesDTO - startoff: "
//					+ startOffset);
//			int zeile = initialisiereEinzelnesDTO(wert, startOffset);
//			System.err.println("ZEILE: " + zeile);
			initialisiereNachHinzufuegenDTO();
		}

	}

	public DataTableObjectErweitert bestimmeLetztesAenderungsDarstellungsObjekt()
	{
		DataTableObjectErweitert re = null;
		Iterator<DataTableObjectErweitert> it = _tabellenObjekte.iterator();
		while (it.hasNext())
		{
			DataTableObjectErweitert dto = it.next();
			if (dto.isAenderungsDarstellung())
			{
//				System.err.println("DTO letzte Zeile: " + dto.getEndOffsetZeile());

				re = dto;
			}
		}

		return re;
	}

	/**
	 * In dieser Methode erfolgt die Auswertung der jeweiligen Zelle, dabei wird
	 * das jeweilige Data-Objekt der Zelle geprft. Ist die Zelle vom Typ:<br>
	 * <ul>
	 * <li>Plain, sind keine weiteren Aktionen notwendig.</li>
	 * <li>List, dann werden die Nachfolger Elemente der Liste ausgewertet und
	 * neue Instanzen der Klasse {@link Zelle} erzeugt und zur Liste der Zelle
	 * hinzugefgt. Die Eltern-Zelle der neu erzeugten Zelle wird automatisch
	 * gesetzt.</li>
	 * <li>Array, dann werden die Nachfolger Elemente des Arrays ausgewertet und
	 * neue Instanzen der Klasse {@link Zelle} erzeugt und zum Array der Zelle
	 * hinzugefgt. Die Elternzellen der neu erzeugten Zellen werden dabei
	 * automatisch gesetzt.</li>
	 * </ul>
	 * 
	 * @param zelle
	 *          {@link Zelle}, die ausgewertet werden soll.
	 */
	private void auswertenZelle(Zelle zelle)
	{
		// System.out.println("Zelle: " + zelle.getWert() + " ISPLAIN: "
		// + zelle.isPlain());

		if (zelle.getWert() instanceof Data)
		{
			Data data = (Data) zelle.getWert();

			LinkedList<Zelle> vorgaenger = zelle.getVorgaenger();

			String liste = "";
			Iterator<Zelle> itVorg = vorgaenger.iterator();
			while (itVorg.hasNext())
			{
				Zelle vorg = itVorg.next();
				liste = liste + "|" + vorg.getDarstellung() + "|";
			}

			/*
			 * Plain
			 */
			if (data.isPlain())
			{
				// Nichts zu tun.
			}

			/*
			 * Liste
			 */
			else if (data.isList())
			{
				@SuppressWarnings("unchecked")
				Iterator it = data.iterator();
				while (it.hasNext())
				{
					Data nextDataInList = (Data) it.next();
					Zelle listZelle = null;
					if (nextDataInList.isArray())
					{
						// if (nextDataInList.asArray().getLength() > 0)
						// {
						// listZelle = new Zelle(nextDataInList);
						// listZelle.setArray(true);
						// } else
						// {
						// initLeereArrayZelle(zelle);
						// }

						listZelle = new Zelle(nextDataInList);
						listZelle.setArray(true);

					} else if (nextDataInList.isList())
					{
						listZelle = new Zelle(nextDataInList);
						listZelle.setListe(true);
					} else if (nextDataInList.isPlain())
					{
						listZelle = new Zelle(nextDataInList);
						listZelle.setPlain(true);
					}

					if (listZelle != null)
					{
						zelle.addToList(listZelle);
						auswertenZelle(listZelle);
					}
				}
			}

			/*
			 * Array
			 */
			else if (data.isArray())
			{

				Array dataArray = data.asArray();
				int laengeArray = dataArray.getLength();

				if (laengeArray > 0)
				{
					for (int i = 0; i < laengeArray; i++)
					{
						Data itemAusArray = dataArray.getItem(i);

						Zelle addZelle = null;
						if (itemAusArray.isList())
						{
							addZelle = new Zelle(itemAusArray);
							addZelle.setListe(true);
						} else if (itemAusArray.isArray())
						{
							if (/* itemAusArray.asArray().getLength() > 0 */true)
							{
								addZelle = new Zelle(itemAusArray);
								addZelle.setArray(true);
							}
						} else if (itemAusArray.isPlain())
						{
							addZelle = new Zelle(itemAusArray);
							addZelle.setPlain(true);
						}

						if (addZelle != null)
						{
							zelle.addToArray(addZelle);
							auswertenZelle(addZelle);
						}
					}
				} else
				{
					initLeereArrayZelle(zelle);
				}
			}

		}

	}

	/**
	 * Fhrt die Initialisierung einer leeren Array-Zelle durch. Es wird hierzu
	 * folgende Strategie verwendet: Es wird ein Pfad aufgebaut, sowohl fr die
	 * Daten im Metamodell (Kopfzeilen) als auch fr die eigentlichen Daten (unter
	 * den Kopfzeilen) - fr die jeweils aktuelle Zelle. Ist der Pfad gleich, kann
	 * die maximale Breite der Nachfolgeknoten der bestimmten Kopfknotens bestimmt
	 * werden und bestimmt damit die Breite der leeren Datenzelle.
	 */
	protected void initLeereArrayZelle(Zelle zelle)
	{
		LinkedList<Knoten> listeArrayKnoten = new LinkedList<Knoten>();

		/*
		 * 1. Bestimmen aller Konten aus dem Kopf, die zu Arrays gehren
		 */
		Iterator<Knoten> itArrayKnotenBestimmen = _listeAllerKnotenDfs.iterator();
		while (itArrayKnotenBestimmen.hasNext())
		{
			Knoten knoten = itArrayKnotenBestimmen.next();
			if (knoten.isArray())
			{
				listeArrayKnoten.add(knoten);
			}
		}

		if (zelle.getWert() instanceof Data)
		{
			Data data = (Data) zelle.getWert();
			String nameData = data.getName();

			/*
			 * Liste von gleich lautenden Knoten
			 */
			LinkedList<Knoten> gleichLautendeKnoten = new LinkedList<Knoten>();

			Iterator<Knoten> it = listeArrayKnoten.iterator();
			while (it.hasNext())
			{
				Knoten pruefKnoten = it.next();
				Object wert = pruefKnoten.getWert();
				if (wert instanceof Attribute)
				{
					Attribute att = (Attribute) wert;
					if (att.getName().equals(nameData))
					{
						gleichLautendeKnoten.add(pruefKnoten);
					}
				}
			}

			Knoten gemerkterVergleichsKnoten = null;

			/*
			 * Pfade der gleich lautenden Knoten mit denen der Zellen vergleichen
			 */
			Iterator<Knoten> itVergleich = gleichLautendeKnoten.iterator();
			while (itVergleich.hasNext())
			{
				Knoten vergleichsKnoten = itVergleich.next();
				Object wert = vergleichsKnoten.getWert();
				if (wert instanceof Attribute)
				{
					/*
					 * Attribute att = (Attribute) wert; System.err.println("NAME: " +
					 * att.getName());
					 */
					if (vergleichePfad(zelle, vergleichsKnoten))
					{
						gemerkterVergleichsKnoten = vergleichsKnoten;
						break;
					}
				}
			}

			/*
			 * Wenn ein Vergleichsknoten bestimmt werden konnte
			 */
			if (gemerkterVergleichsKnoten != null)
			{
				int breite = gemerkterVergleichsKnoten.bestimmeKnotenOhneNachfolgerRek(
						gemerkterVergleichsKnoten).size();

				Zelle leereZelle = new Zelle("");
				leereZelle.setPlain(true);
				leereZelle.setBreite(breite);
				zelle.addToArray(leereZelle);
				// System.err.println("Leere Zelle ergnzt, Breite: " + breite);
			}

		}

	}

	/**
	 * Vergleicht, ob Zelle und Knoten im gleichen Pfad liegen, wenn es sich bei
	 * den beiden um Arrays handelt.
	 * 
	 * @param zelle
	 *          Zu berprfende Zelle.
	 * @param knoten
	 *          Zu berprfender Knoten.
	 * @return Liefert <code>true</code>, wenn Zelle und Knoten im gleichen Pfad
	 *         liegen, sonst <code>false</code>.
	 */
	private boolean vergleichePfad(Zelle zelle, Knoten knoten)
	{
		boolean re = false;

		if (zelle.isArray() && knoten.isArray())
		{
			LinkedList<Zelle> pfadZelle = zelle.getVorgaenger();
			LinkedList<Knoten> pfadKnoten = knoten.getVorgaenger();

			// System.err.println("PfadZelle Elmente: " + pfadZelle.size());
			// System.err.println("PfadKnoten Elemente: " + pfadKnoten.size());

			String zellPfad = "";
			Iterator<Zelle> itZelle = pfadZelle.iterator();
			while (itZelle.hasNext())
			{
				String add = "";
				Zelle pruefZelle = itZelle.next();
				if (pruefZelle.getWert() instanceof Data)
				{
					Data dataPruefZelle = (Data) pruefZelle.getWert();

					add = dataPruefZelle.getName();
					if (dataPruefZelle.isArray())
					{
						add = add + " [Array]";
					}
				}

				// Wenn im Pfad keine Zahlen vorhanden sind (bezeichnen die Nummer
				// des aktuellen Array-Elementes im Array, dann wird sie zum Pfad
				// hinzugefgt. Auf diesee Weise lassen sich der Pfad der Kopfzeilen
				// (reprsentieren das Modell) und der der Datenzellen (reprsentieren
				// das Data) auf einander angleichen zur Bestimmung der maximalen Breite
				// einer leeren Zelle. Evtl. gibt es hier eine bessere Lsung?!?

				try
				{
					Integer.parseInt(add);
				} catch (NumberFormatException e)
				{
					zellPfad = zellPfad + " | " + add;
				}

			}

			String knotenPfad = "";
			Iterator<Knoten> itKnoten = pfadKnoten.iterator();
			while (itKnoten.hasNext())
			{
				Knoten pruefKnoten = itKnoten.next();
				String add = "";

				if (pruefKnoten.getWert() instanceof Attribute)
				{
					add = ((Attribute) pruefKnoten.getWert()).getName();

					if (pruefKnoten.isArray())
					{
						add = add + " [Array]";
					}
				} else if (pruefKnoten.getWert() instanceof AttributeGroup)
				{
					// add = ((AttributeGroup) pruefKnoten.getWert()).getName() + "
					// [ATG]";
					add = ((AttributeGroup) pruefKnoten.getWert()).getPid();
				}

				knotenPfad = knotenPfad + " | " + add;

			}

			// System.err.println("Pfad Zellen: " + zellPfad);
			// System.err.println("Pfad Knoten: " + knotenPfad);

			if (zellPfad.equals(knotenPfad))
			{
				// System.err.println("==>Pfad ist gleich!");
				re = true;
			}
		}

		return re;
	}

	/**
	 * Bestimmt das {@link DataTableObjectErweitert}, welches sich im angegebenen
	 * Zeilenbereich befindet.
	 * 
	 * @param zeilenNummer
	 *          Nummer der Zeile, zu der das zugehrige
	 *          {@link DataTableObjectErweitert} bestimmt werden soll.
	 * 
	 * @return Liefert das zur Zeile zugehrige {@link DataTableObjectErweitert}
	 *         oder <code>null</code>, wenn zu der angegebenen Zeile kein
	 *         {@link DataTableObjectErweitert} existiert.
	 */
	public DataTableObjectErweitert getDtoImZeilenBereich(int zeilenNummer)
	{
		/*
		 * Bestimmen des DataTableObjectErweitert im entsprechenden Zeilenbereich
		 */
		DataTableObjectErweitert dtoImZeilenbereich = null;
		Iterator<DataTableObjectErweitert> itDto = _tabellenObjekte.iterator();
		while (itDto.hasNext())
		{
			DataTableObjectErweitert dataTableObjectErweitert = itDto.next();

			// if ((_darstellungsArt.equals(EnumDarstellungsArt.zustandsDarstellung))
			// || ((_darstellungsArt
			// .equals(EnumDarstellungsArt.aenderungsDarstellung)) &&
			// (dataTableObjectErweitert
			// .isAenderungsDarstellung())))
			// {

			int startOffsetZeile = dataTableObjectErweitert.getStartOffsetZeile();
			int endOffsetZeile = dataTableObjectErweitert.getEndOffsetZeile();
			// System.err.println("Zeile (" + zeilenNummer + ") StartOffset: "
			// + startOffsetZeile + " EndOffset: " + endOffsetZeile);
			if ((zeilenNummer >= startOffsetZeile)
					&& (zeilenNummer <= endOffsetZeile))
			{
				// System.err.println(">=<=");
				dtoImZeilenbereich = dataTableObjectErweitert;
				// System.out.println("Zeile (" + zeilenNummer + ")StartOffset: "
				// + startOffsetZeile + " EndOffset: " + endOffsetZeile);
				break;
			}
			// }
		}

		return dtoImZeilenbereich;
	}

	@Override
	public KTableCellEditor doGetCellEditor(int col, int row)
	{
		if (isFixedCell(col, row))
		{
			return null;
		} else
		{
			return new KTableCellEditorText();
		}
	}

	@Override
	public KTableCellRenderer doGetCellRenderer(int col, int row)
	{
		if (isFixedCell(col, row))
		{
//			return KTableCellRenderer.defaultRenderer;
			return _fixedCellRendererDambach;
		} else
		{
			if (_cellRenderer == null)
			{
				_cellRenderer = new CellRenderer();
			}
			return _cellRenderer;
		}
	}

	@Override
	public int doGetColumnCount()
	{
		int re = 4;

		if (_wurzelKopfZeile != null)
		{
			re = _wurzelKopfZeile.bestimmeKnotenOhneNachfolger().size() + 3;
		}

		return re;
	}

	/**
	 * Diese Methode liefert den Inhalt einer Zelle, an Hand der Parameter Spalte
	 * und Zeile.
	 * 
	 * @param col
	 *          Spalte
	 * @param row
	 *          Zeile
	 * @return Objekt, das i.d. Zelle dargestellt werden soll.
	 */
	public Object doGetContentAt(int col, int row)
	{

		Object re = ""; // Leer Text

		if (col < _spaltenOffset)
		{
			/*
			 * Bereich der Kopfzeilen
			 */
			if (row < getFixedHeaderRowCount())
			{
				/*
				 * Spalten Art, Zeit & Objekt
				 */
				switch (col)
				{
				case 0:
					re = "Art";
					break;
				case 1:
					re = "Zeit";
					break;
				case 2:
					re = "Objekt";
					break;
				}
			} else if (row >= getFixedHeaderRowCount())
			{
				re = "_";

				int zeile = row - getFixedHeaderRowCount();

				/*
				 * Bestimmen des DataTableObjectErweitert im entsprechenden
				 * Zeilenbereich
				 */
				DataTableObjectErweitert dtoImZeilenbereich = getDtoImZeilenBereich(zeile);

				/*
				 * Wenn ein entsprechendes Objekt gefunden wurde, dann erfolgt die
				 * Auswertung, um was fr einen ArchiveDataKind es sich handelt. Diese
				 * werden genau wie im GTM dargestellt.
				 */
				if (dtoImZeilenbereich != null)
				{
					switch (col)
					{
					case 0:
						ArchiveDataKind dataKind = dtoImZeilenbereich.getDataKind();
						if (dataKind == ArchiveDataKind.ONLINE)
						{
							re = "OA";

						} else if (dataKind == ArchiveDataKind.ONLINE_DELAYED)
						{
							re = "ON";
						} else if (dataKind == ArchiveDataKind.REQUESTED)
						{
							re = "NA";
						} else if (dataKind == ArchiveDataKind.REQUESTED_DELAYED)
						{
							re = "NN";
						} else
						{
							re = "??";
						}
						break;
					case 1:
						DateFormat dfmt = new SimpleDateFormat("dd.MM.yyyy HH.mm.ss,SSS");
						re = dfmt.format(new Date(dtoImZeilenbereich.getDataTime()));

						break;
					case 2:
						re = dtoImZeilenbereich.getObject().getNameOrPidOrId();
						break;
					}
				}

			}
		} else if (col >= _spaltenOffset)
		{
			/*
			 * Kopfzeilen
			 */
			if (row < getFixedHeaderRowCount())
			{
				int spalte = col - _spaltenOffset;

				LinkedList<Knoten> bestimmeKnotenAufEbene = _wurzelKopfZeile
						.bestimmeKnotenAufEbene(row);
				Iterator<Knoten> it = bestimmeKnotenAufEbene.iterator();
				while (it.hasNext())
				{
					Knoten knoten = it.next();

					if (knoten.istInBereich(spalte))
					{
						re = knoten.getSpaltenBezeichnung();
					}
				}
			} else if (row >= getFixedHeaderRowCount())
			{

				/*
				 * Datenbereich
				 */
				int spalte = col - _spaltenOffset;
				int zeile = row - getFixedHeaderRowCount();
				// System.out.println("=> ZEILE: " + zeile + "SPALTE: " + spalte
				// + " "
				// + _plainZellenData.size());

				/*
				 * Bestimmen des DataTableObjectErweitert im entsprechenden
				 * Zeilenbereich
				 */
				DataTableObjectErweitert dtoImZeilenbereich = getDtoImZeilenBereich(zeile);

				if (dtoImZeilenbereich != null)
				{
					/*
					 * Iterator ber alle Plain-Zellen. Nur diese werden dargestellt.
					 */
					Iterator<Zelle> it = dtoImZeilenbereich.getPlainZellen().iterator();
					while (it.hasNext())
					{
						Zelle zelle = it.next();

						/*
						 * int plainZellenZeile = zeile -
						 * dtoImZeilenbereich.getStartOffsetZeile();
						 * 
						 * System.out.println("ZEILE: " + plainZellenZeile + " SPALTE: " +
						 * spalte);
						 */

						if (zelle.istInBereich(zeile
								- dtoImZeilenbereich.getStartOffsetZeile(), spalte))
						{
							re = zelle.getDarstellung();
							// System.out.println("\t=>" + re);
						}
					}

				} else
				{
					/*
					 * Leeren Inhalt setzen
					 */
					re = "";
				}

			}

		}

		// System.out.println("doGetContentAt => ROW: " + row + " COL: " + col);
		// System.out.println("\tcontent = " + re);

		return re;
	}

	protected Knoten hatVorgaengerZelleOhneNachfolger(int ebene, int spalte)
	{
		Knoten re = null;
		if (ebene > 0)
		{
			LinkedList<Knoten> knotenAufEbene = _wurzelKopfZeile
					.bestimmeKnotenAufEbene(ebene - 1);
			Iterator<Knoten> it = knotenAufEbene.iterator();
			while (it.hasNext())
			{
				Knoten knoten = it.next();
				if (!knoten.hatNachfolger())
				{
					if (knoten.istInBereich(spalte))
					{
						re = knoten;
						break;
					}
				}
			}
			if (re == null)
			{
				re = hatVorgaengerZelleOhneNachfolger(ebene - 1, spalte);
			}

		}
		return re;
	}

	/**
	 * Mit Hilfe dieser Methode werden die Zellenbereiche bestimmt, die zu einer
	 * Zelle gehren.<br>
	 * Die Fragestellung lautet: Zu welchem Zellenbereich gehrt die Zelle in
	 * Spalte X und Zeile Y? <br>
	 * Als Antwort wird die linke obere Zelle des entsprechenden Bereiches
	 * zurckgeliefert.<br>
	 * KTable erzeugt hieraus eine entsprechend groe, zusammenhngende Zelle.
	 */
	public Point doBelongsToCell(int col, int row)
	{
		// System.out.println("doBelongsToCell -> ROW: " + row + " COL: " + col);

		Point re = null;

		/*
		 * Erste Spalte im Kopf == "Objekt"
		 */
		if ((col == 0) && (row < getFixedRowCount()))
			re = new Point(col, 0);

		/*
		 * Zweite Spalte im Kopf == "Zeit"
		 */
		else if ((col == 1) && (row < getFixedRowCount()))
			re = new Point(col, 0);

		/*
		 * Dritte Spalte im Kopf == "Objekt"
		 */
		else if ((col == 2) && (row < getFixedRowCount()))
			re = new Point(col, 0);

		/*
		 * Kopfzeilte der Attributgruppe
		 */
		else if (col > 2 && row <= getFixedHeaderRowCount())
		{
			LinkedList<Knoten> alleKnotenDfs = _listeAllerKnotenDfs;
			Iterator<Knoten> it = alleKnotenDfs.iterator();
			while (it.hasNext())
			{
				Knoten knoten = it.next();

				if (knoten.istInBereich(row, col - _spaltenOffset))
				{
					re = new Point(knoten.getVonX() + _spaltenOffset, knoten.getVonY());
				}
			}

		}

		/*
		 * Eigentliche Daten
		 */
		else if (col > 2 && row > getFixedHeaderRowCount())
		{

			/*
			 * Bestimmen des DataTableObjectErweitert im entsprechenden Zeilenbereich
			 */
			int zeilenNummer = row - getFixedHeaderRowCount();
			// System.out.println("ROW: " + row + " HEADER: " +
			// getFixedHeaderRowCount() + "ZEILE: " + zeilenNummer);

			DataTableObjectErweitert dtoImZeilenbereich = getDtoImZeilenBereich(zeilenNummer);
			// System.out.println("DA");

			if (dtoImZeilenbereich != null)
			{
				/*
				 * Bearbeiten aller Plain-Zellen
				 */
				// System.err.println("Anzahl Plain Zellen: "
				// + dtoImZeilenbereich.getPlainZellen().size());
				Iterator<Zelle> it = dtoImZeilenbereich.getPlainZellen().iterator();
				while (it.hasNext())
				{
					Zelle zelle = it.next();
					// System.out.println("Z: " + zelle.getWert());

					int zeile = row - getFixedHeaderRowCount()
							- dtoImZeilenbereich.getStartOffsetZeile();
					int spalte = col - _spaltenOffset;

					if (zelle.istInBereich(zeile, spalte))
					{
						int pointCol = zelle.getStartX() + _spaltenOffset;

						int pointRow = zelle.getStartY() + getFixedHeaderRowCount()
								+ dtoImZeilenbereich.getStartOffsetZeile();

						re = new Point(pointCol, pointRow);
					} else
					{
						// System.out.println("ZELLE NICHT IN BEREICH");
					}
				}

			} else
			// Trick 17 <- KTable interpretiert anscheinend die erste Datenzeile
			// nicht immer Richtig. Deshalb ist es auch nicht mglich, eine Zeile ber
			// die gesamte Breite als erste Zeile zu Bringen, die nur aus einer Zelle
			// besteht und "keine Quelle, keine Daten" anzeigt.
			{
				/*
				 * Anscheinend verrechnet sich KTable bei nur 1er echten Datenzeile
				 * dramatisch. Es wird die entsprechende Zeile als mit 3 oder 4
				 * angesprochen. Das ist schade, denn so kann bei fehlendem Sender
				 * nichts sinnvolles dargestellt werden.
				 */
			}

		}

		/*
		 * Informationen zu den Objekten
		 */
		else if (col <= 2 && row > getFixedHeaderRowCount())
		{
			/*
			 * Bestimmen des DataTableObjectErweitert im entsprechenden Zeilenbereich
			 */
			DataTableObjectErweitert dtoImZeilenbereich = null;
			Iterator<DataTableObjectErweitert> itDto = _tabellenObjekte.iterator();
			while (itDto.hasNext())
			{
				DataTableObjectErweitert dataTableObjectErweitert = itDto.next();
				int startOffsetZeile = dataTableObjectErweitert.getStartOffsetZeile();
				int endOffsetZeile = dataTableObjectErweitert.getEndOffsetZeile();

				if ((row >= startOffsetZeile + getFixedHeaderRowCount())
						&& (row <= endOffsetZeile + getFixedHeaderRowCount()))
				{
					// System.out.println("Start: " + startOffsetZeile + " Ende:
					// " +
					// endOffsetZeile);
					dtoImZeilenbereich = dataTableObjectErweitert;
					break;
				}
			}

			if (dtoImZeilenbereich != null)
			{
				int startOffsetZeile = dtoImZeilenbereich.getStartOffsetZeile();
				re = new Point(col, startOffsetZeile + getFixedHeaderRowCount());
			}

		}

		// System.out.println("POINT: " + re);

		return re;
	}

	public boolean pruefeImBereichNaechsteZeilen(int zeile, int von, int bis)
	{
		boolean re = false;
		LinkedList<Knoten> knotenAufEbene = _wurzelKopfZeile
				.bestimmeKnotenAufEbene(zeile);
		Iterator<Knoten> it = knotenAufEbene.iterator();
		while (it.hasNext())
		{
			Knoten knoten = it.next();
			if ((knoten.istInBereich(von)) || (knoten.istInBereich(bis)))
			{
				re = true;
				break;
			}
		}

		return re;
	}

	public int berechneAnzahlZeilen()
	{
		int rowCount = _tiefsteEbeneKopfzeile + 1;
		Iterator<DataTableObjectErweitert> it = _tabellenObjekte.iterator();
		while (it.hasNext())
		{
			DataTableObjectErweitert tabellenObjekt = it.next();

			if (_darstellungsArt.equals(EnumDarstellungsArt.aenderungsDarstellung))
			{
				if (tabellenObjekt.isAenderungsDarstellung())
				{
					rowCount = rowCount + tabellenObjekt.getTiefsteEbene();
				}
			} else
			{
				rowCount = rowCount + tabellenObjekt.getTiefsteEbene();
			}
		}

		return rowCount;
	}

	@Override
	public int doGetRowCount()
	{
		// int rowCount = _tiefsteEbeneKopfzeile + 1;
		// Iterator<DataTableObjectErweitert> it = _tabellenObjekte.iterator();
		// while (it.hasNext())
		// {
		// DataTableObjectErweitert tabellenObjekt = it.next();
		// rowCount = rowCount + tabellenObjekt.getTiefsteEbene() + 1;
		// }
		//
		// return rowCount;

		// System.out.println("doGetRowCount := " + _rowCount);

		return _rowCount;
	}

	@Override
	public void doSetContentAt(int col, int row, Object value)
	{
		System.out.println("==>Setze - Spalte: " + col + " Zeile: " + row
				+ " Wert: " + value);

		/*
		 * Vorgehensweise: -Suchen des zugehrigen DataTableObject an Hand der
		 * Start- und Endzeile -Anpassen der Zeilen an Hand des Offsets der
		 * Startzeile -Zugehriges Objekt der Zelle suchen -Wert anpassen
		 * -Dirty-Flag setzen
		 */

	}

	@Override
	public int getInitialColumnWidth(int column)
	{
		int re = 50;

		switch (column)
		{
		case 0:
			re = 40;
			break;
		case 1:
			re = 150;
			break;
		case 2:
			re = 150;
			break;
		}

		return re;
	}

	@Override
	public int getInitialRowHeight(int row)
	{
		return 20;
	}

	@Override
	public int getFixedHeaderColumnCount()
	{
		return 3;
	}

	@Override
	public int getFixedHeaderRowCount()
	{
		return _fixedHeaderRowCount;
	}

	@Override
	public int getFixedSelectableColumnCount()
	{
		return 0;
	}

	@Override
	public int getFixedSelectableRowCount()
	{
		return 0;
	}

	@Override
	public int getRowHeightMinimum()
	{
		return 15;
	}

	@Override
	public boolean isColumnResizable(int col)
	{
		return true;
	}

	@Override
	public boolean isRowResizable(int row)
	{
		return false;
	}

	@Override
	public FaltStatusSpalten getFaltStatusSpalten()
	{
		return _faltStatusSpalten;
	}

	/**
	 * Liefert die Wurzel der Kopfzeile.
	 * 
	 * @return Wurzel der Kopfzeile.
	 */
	public Knoten getWurzelKopfzeile()
	{
		return _wurzelKopfZeile;
	}

}
