/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.login;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.MessageBox;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.rw.rw.Activator;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.CommunicationError;
import de.bsvrz.dav.daf.main.ConnectionException;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.InconsistentLoginException;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.ConfigurationObjectType;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.dav.daf.main.config.SystemObjectType;

/**
 * Diese Klasse liefert alle Informationen zur Urlasserverbindung und ist in der
 * Lage, eine Liste von {@link ResultData}s zu versenden. Diese werden um die
 * Urlasser-Informationen (Ursache und Veranlasser) erweitert.
 * 
 * @author HofmannJ
 * 
 */
public class UrlasserObjekt implements ClientSenderInterface {
	/**
	 * Verweis auf den {@link UrlasserDialog} zum Auslesen der notwendigen
	 * Felder.
	 */
	private UrlasserDialog _urlasserDialog = null;

	/**
	 * IP-Adresse des DaV
	 */
	private String _ipDaV = null;

	/**
	 * Port des DaV
	 */
	private int _portDaV = -1;

	/**
	 * Verbindung zum DaV
	 */
	private ClientDavConnection _davVerbindung = null;

	/**
	 * Dieses Flag zeigt an, ob eine bestehende DaV-Verbindung benutzt wird.
	 */
	private boolean _benutzeBestehendeDaVVerbindung = false;

	/**
	 * Timeout, gibt die max. Wartezeit auf die Sendesteuerung an.
	 */
	private static final long _timeoutSendesteuerung = 15000;

	/**
	 * Objekt zum Synchronisieren an Hand der Sendesteuerung.
	 */
	private Boolean _sendeSteuerung = new Boolean(true);

	/**
	 * UrlasserInfo
	 */
	private UrlasserInfo _urlasserInfo = null;

	/**
	 * Konstruktor, bernimmt Ursache, Veranlasser sowie die Verbindung zum DaV.
	 * 
	 * @param ursache
	 *            Ursache
	 * @param veranlasser
	 *            Veranlasser
	 * @param davVerbindung
	 *            Verbindung zum DaV
	 */
	public UrlasserObjekt(UrlasserDialog urlasserDialog) {
		reInit(urlasserDialog);
	}

	public UrlasserObjekt(UrlasserInfo urlasserInfo) {
		this._urlasserInfo = urlasserInfo;
	}

	/**
	 * Initialisiert diese Klasse mit Hilfe des {@link UrlasserDialog}s. Es
	 * wird Benutzer, Passwort, Ursache sowie Veranlasser ausgelesen.
	 * 
	 * @param urlasserDialog
	 *            UrlasserDialog, aus dem die Daten ausgelesen werden sollen.
	 */
	protected void reInit(UrlasserDialog urlasserDialog) {
		this._urlasserDialog = urlasserDialog;
		initHoleDatenAusDialog();
	}

	/**
	 * Holt die Daten aus dem {@link UrlasserDialog}.
	 */
	private void initHoleDatenAusDialog() {
		this._urlasserInfo = new UrlasserInfo(_urlasserDialog.getBenutzer(),
				_urlasserDialog.getPasswort(), _urlasserDialog.getUrsache(),
				_urlasserDialog.getVeranlasser());

	}

	/**
	 * Initialisiert die Verbindung zum DaV
	 */
	public boolean baueDaVVerbindungAuf() {
		boolean erfolgreich = false;

		/*
		 * Wenn eine DaV-Verbindung aktiv ist, dann deren Verbindungsdaten (IP,
		 * PORT) bernehmen.
		 */
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv()) {
			if (_urlasserInfo != null) {
				// if ((_benutzer != null) && (_passwort != null))
				if ((_urlasserInfo.getBenutzer() != null)
						&& (_urlasserInfo.getPasswort() != null)) {
					if (DaVVerbindung.getInstanz().getDaVVerbindung()
							.checkLoggedUserNameAndPassword(
									_urlasserInfo.getBenutzer(),
									_urlasserInfo.getPasswort())) {
						_davVerbindung = (ClientDavConnection) DaVVerbindung
								.getInstanz().getDaVVerbindung();
						_benutzeBestehendeDaVVerbindung = true;
						erfolgreich = true;
					} else {
						_ipDaV = DaVVerbindung.getInstanz().getDaVIp();
						_portDaV = DaVVerbindung.getInstanz().getDaVPort();

						try {
							/*
							 * Die ClientDavParameters werden mit Hilfe der
							 * Kommandozeilenparameter initialisiert, so dass
							 * die gewohnten Parameter fr DaV-Applikationen zur
							 * Verfgung stehen.
							 */
							String[] commandLineArgs = Platform
									.getCommandLineArgs();
							LinkedList<String> lArgs = new LinkedList<String>();

							for (String arg : commandLineArgs) {
								lArgs.add(arg);
							}

							String[] aArgs = new String[lArgs.size()];
							int i = 0;
							for (String arg : lArgs) {
								aArgs[i] = arg;
								i++;
							}

							ClientDavParameters cdp = new ClientDavParameters(
									aArgs);

							_davVerbindung = new ClientDavConnection(cdp);

							if ((_ipDaV != null) && (_portDaV >= 0)
									&& (_portDaV <= 65535)) {
								_davVerbindung.getClientDavParameters()
										.setDavCommunicationAddress(_ipDaV);
								_davVerbindung
										.getClientDavParameters()
										.setDavCommunicationSubAddress(_portDaV);

								try {
									_davVerbindung.connect();

									try {
										// _davVerbindung.login(_benutzer,
										// _passwort);
										_davVerbindung.login(_urlasserInfo
												.getBenutzer(), _urlasserInfo
												.getPasswort());
										erfolgreich = true;
									} catch (InconsistentLoginException e) {
										String meldung = "UrlasserObjekt: Der Benutzer "
												+ _urlasserInfo.getBenutzer()
												+ " konnte nicht am DaV authentifiziert werden.";

										Activator.getDefault().getLog().log(
												new Status(Status.WARNING,
														Activator.PLUGIN_ID,
														Status.OK, meldung, e));

										generiereMeldungFehler(
												"Keine Authentifizierung mglich",
												meldung);

										_davVerbindung = null;
									}

								} catch (CommunicationError e) {
									String meldung = "UrlasserObjekt: Fehler bei der initialen Kommunikation mit dem DaV.";

									Activator.getDefault().getLog().log(
											new Status(Status.WARNING,
													Activator.PLUGIN_ID,
													Status.OK, meldung, e));

									generiereMeldungFehler(
											"Fehler initiale Kommunikation",
											meldung + "\n\t=> " + e.toString());

									_davVerbindung = null;
								} catch (ConnectionException e) {
									String meldung = "UrlasserObjekt: Die Verbindung zum DaV konnte nicht aufgebaut werden";

									Activator.getDefault().getLog().log(
											new Status(Status.WARNING,
													Activator.PLUGIN_ID,
													Status.OK, meldung, e));

									generiereMeldungFehler("Verbindungsfehler",
											meldung + "\n\t=> " + e.toString());

									_davVerbindung = null;
								}

							} else {
								String meldung = "UrlasserObjekt: Konnte IP und Port nicht aus der aktiven DaV-Verbindung extrahieren.";

								Activator.getDefault().getLog().log(
										new Status(Status.WARNING,
												Activator.PLUGIN_ID, Status.OK,
												meldung, null));

								generiereMeldungFehler(
										"IP & Port nicht verfgbar", meldung);

								_davVerbindung = null;
							}

						} catch (MissingParameterException e) {
							String meldung = "UrlasserObjekt: Es fehlen Parameter zur Initialisierung der DaV-Verbindung";

							Activator.getDefault().getLog().log(
									new Status(Status.WARNING,
											Activator.PLUGIN_ID, Status.OK,
											meldung, e));

							generiereMeldungFehler("Fehlende Parameter",
									meldung + "\n\t=> " + e.toString());

						}
					}
				}
			}

		} else {
			String meldung = "Es ist zur Zeit keine Benutzerverbindung zum DaV aktiv. "
					+ "Kann die IP sowie den Port nicht extrahieren.";

			Activator.getDefault().getLog().log(
					new Status(Status.WARNING, Activator.PLUGIN_ID, Status.OK,
							meldung, null));

			generiereMeldungFehler("Keine DaV-Verbindung aktiv", meldung);
		}

		return erfolgreich;
	}

	/**
	 * Trennt die Verbindung zum DaV, wenn eine neue DaV-Verbindung fr einen
	 * anderen Benutzer als den angemeldeten Benutzer aufgebaut wurde. Ansonsten
	 * bleibt die DaV-Verbindung bestehen.
	 */
	private void trenneDaVVerbindung() {
		if (_davVerbindung != null) {
			if (!_benutzeBestehendeDaVVerbindung) {
				_davVerbindung.disconnect(false, null);
			}
		}
	}

	/**
	 * Sendet die Daten. Wurden alle Daten gesendet, wird die Verbindung zum DaV
	 * getrennt, falls eine neue Urlasserverbindung geffnet wurde.
	 * 
	 * @param resultDatas
	 *            {@link Collection} von {@link ResultData}s, die gesendet
	 *            werden sollen.
	 * @return Liste von {@link ResultData}s, die nicht gesendet werden
	 *         konnten.
	 */
	public LinkedList<ResultData> sendeDaten(Collection<ResultData> resultDatas) {
		/**
		 * Liste der ResultDatas, die nicht gesendet werden konnten!
		 */
		LinkedList<ResultData> fehlgeschlagen = new LinkedList<ResultData>();

		if ((_davVerbindung != null) && (resultDatas != null)) {
			Iterator<ResultData> it = resultDatas.iterator();
			while (it.hasNext()) {
				ResultData resultData = it.next();

				SystemObject systemObject = resultData.getObject();
				DataDescription dataDescription = resultData
						.getDataDescription();

				/*
				 * Kopie erzeugen
				 */
				Data dataCopy = resultData.getData().createModifiableCopy();

				/*
				 * Kopie modifizieren (Urlasser hinzufgen)
				 */
				Data itemUrlasser = dataCopy.getItem("Urlasser");

				/*
				 * Prfen, ob die Daten ein Urlasserelement beinhalten.
				 */
				if (itemUrlasser != null) {
					// itemUrlasser.getReferenceValue("BenutzerReferenz").setSystemObject(
					// bestimmeSystemObjektEinesBenutzers(_benutzer));
					// itemUrlasser.getTextValue("Ursache").setText(_ursache);
					// itemUrlasser.getTextValue("Veranlasser").setText(_veranlasser);
					itemUrlasser
							.getReferenceValue("BenutzerReferenz")
							.setSystemObject(
									bestimmeSystemObjektEinesBenutzers(_urlasserInfo
											.getBenutzer()));
					itemUrlasser.getTextValue("Ursache").setText(
							_urlasserInfo.getUrsache());

					itemUrlasser.getTextValue("Veranlasser").setText(
							_urlasserInfo.getVeranlasser());
				}

				/*
				 * Neues ResultData zusammenbauen
				 */
				// ResultData neuesResultData = new ResultData(systemObject,
				// dataDescriptionNeu, resultData.getDataTime(), dataCopy);
				ResultData neuesResultData = new ResultData(systemObject,
						dataDescription, resultData.getDataTime(), dataCopy);

				/*
				 * Anmelden zum Senden
				 */
				// anmeldenSenden(systemObject, dataDescriptionNeu);
				anmeldenSenden(systemObject, dataDescription);

				/*
				 * Warten bis die Sendesteuerung zugeteilt wird bis zum
				 * maximalen Timeout.
				 */
				try {
					synchronized (_sendeSteuerung) {
						_sendeSteuerung.wait(_timeoutSendesteuerung);
					}

					try {
						_davVerbindung.sendData(neuesResultData);
					} catch (DataNotSubscribedException e) {
						String message = getClass().getName()
								+ "::sendeDaten Es fand keine Subskription fr die Daten statt!";

						Activator.getDefault().getLog().log(
								new Status(Status.WARNING, Activator.PLUGIN_ID,
										Status.OK, message, e));

						if (!fehlgeschlagen.contains(resultData)) {
							fehlgeschlagen.add(resultData);
						}

					} catch (SendSubscriptionNotConfirmed e) {
						String message = getClass().getName()
								+ "::sendeDaten Die Subskription zum Senden wurde nicht besttigt!";

						Activator.getDefault().getLog().log(
								new Status(Status.WARNING, Activator.PLUGIN_ID,
										Status.OK, message, e));

						if (!fehlgeschlagen.contains(resultData)) {
							fehlgeschlagen.add(resultData);
						}
					}

				} catch (InterruptedException e) {
					String message = getClass().getName()
							+ "::sendeDaten Das Senden der Daten wurde unterbrochen "
							+ "bzw. die SendeSteuerung wurde nach "
							+ _timeoutSendesteuerung + " ms noch "
							+ "nicht zugeteilt!";

					Activator.getDefault().getLog().log(
							new Status(Status.WARNING, Activator.PLUGIN_ID,
									Status.OK, message, e));

					if (!fehlgeschlagen.contains(resultData)) {
						fehlgeschlagen.add(resultData);
					}
				}

				// abmeldenSenden(systemObject, dataDescriptionNeu);
				abmeldenSenden(systemObject, dataDescription);
			}

		} else {
			System.out.println("Keine DaV-Verbindung bestimmbar...");
		}

		/*
		 * Trennen der DaV-Verbindung
		 */
		trenneDaVVerbindung();

		return fehlgeschlagen;
	}

	/**
	 * Anmeldung zum Senden
	 */
	private void anmeldenSenden(SystemObject systemObject,
			DataDescription dataDescription) {
		if ((systemObject != null) && (dataDescription != null)
				&& (_davVerbindung != null)) {
			SenderRole senderRole = SenderRole.sender();
			try {
				_davVerbindung.subscribeSender(this, systemObject,
						dataDescription, senderRole);

			} catch (OneSubscriptionPerSendData e) {
				String meldung = getClass().getName()
						+ " konnte nicht zum Senden angemeldet werden";

				Activator.getDefault().getLog().log(
						new Status(Status.WARNING, Activator.PLUGIN_ID,
								Status.OK, meldung, e));

				generiereMeldungFehler("Sendeanmeldung fehlgeschlagen", meldung
						+ "\n\t=> " + e.toString());
			}
		} else {
			String meldung = getClass().getName()
					+ ":: Es ist keine Anmeldung zum Senden mglich:\n";
			if (systemObject == null) {
				meldung = meldung + "- SystemObject == null\n";
			}
			if (dataDescription == null) {
				meldung = meldung + "- DataDescription == null\n";
			}
			if (_davVerbindung == null) {
				meldung = meldung + "- DaV-Verbindung == null\n";
			}

			Activator.getDefault().getLog().log(
					new Status(Status.WARNING, Activator.PLUGIN_ID, Status.OK,
							meldung, null));

			generiereMeldungFehler("Sendeanmeldung fehlgeschlagen", meldung);

		}

	}

	/**
	 * Abmeldung vom Senden
	 */
	private void abmeldenSenden(SystemObject systemObject,
			DataDescription dataDescription) {
		if ((_davVerbindung != null) && (dataDescription != null)) {
			_davVerbindung.unsubscribeSender(this, systemObject,
					dataDescription);
		} else {
			String message = getClass().getName()
					+ " konnte nicht vom Senden abgemeldet werden\n\t=> "
					+ "Verbindung zum DaV oder die DataDescription == null!";

			Activator.getDefault().getLog().log(
					new Status(Status.INFO, Activator.PLUGIN_ID, message));

		}
	}

	@Override
	public void dataRequest(SystemObject object,
			DataDescription dataDescription, byte state) {
		if (state == 0) {
			synchronized (_sendeSteuerung) {
				_sendeSteuerung.notify();
			}
		}

	}

	@Override
	public boolean isRequestSupported(SystemObject object,
			DataDescription dataDescription) {
		/*
		 * Liefert der Sendesteuerung true, denn Daten knnen gesendet werden.
		 */
		return true;
	}

	/**
	 * Generiert eine grafische Fehlermeldung und zeigt diese an.
	 * 
	 * @param titel
	 *            Titel der Fehlermeldung
	 * @param meldung
	 *            Mitteilung der Fehlermeldung
	 */
	private void generiereMeldungFehler(String titel, String meldung) {
		MessageBox mb = new MessageBox(PlatformUI.getWorkbench().getDisplay()
				.getActiveShell(), SWT.OK | SWT.ICON_ERROR);
		mb.setText(titel);
		mb.setMessage(meldung);
		mb.open();
	}

	/**
	 * Bestimmt die PID des zu einem Benutzer zugehrigen Objektes vom Typ
	 * <code>typ.benutzer</code>.
	 * 
	 * @param benutzer
	 *            Name des Benutzer-Objektes
	 * @return {@link SystemObject} des Benutzers oder <code>null</code>,
	 *         wenn kein entsprechender Benutzer existiert.
	 */
	private SystemObject bestimmeSystemObjektEinesBenutzers(String benutzer) {
		SystemObject reSo = null;

		List<SystemObject> bestimmeObjekte = bestimmeObjekte("typ.benutzer");
		Iterator<SystemObject> it = bestimmeObjekte.iterator();
		while (it.hasNext()) {
			SystemObject so = it.next();
			if (so.getName().equals(benutzer)) {
				reSo = so;
				break;
			}
		}

		return reSo;
	}

	/**
	 * Methode zum Bestimmen der Objekte, die zu einer bestimmten PID gehren
	 * (z.B. typ.de)
	 * 
	 * @param objPid
	 *            Pid der Objekttypen
	 * @return Liste mit den Systemobjekten
	 */
	private List<SystemObject> bestimmeObjekte(String objPid) {
		List<SystemObject> objekte = new ArrayList<SystemObject>();

		// Systemobjekt erzeugen

		SystemObjectType typeSysObj = null;

		typeSysObj = _davVerbindung.getDataModel().getType(objPid);

		if (typeSysObj == null)
			return objekte;

		if (typeSysObj.isConfigurating()) {
			typeSysObj = (ConfigurationObjectType) typeSysObj;
		} else {
			typeSysObj = (DynamicObjectType) typeSysObj;
		}

		// Elemente dieses Systemobjekts einlesen

		List<SystemObject> typobjekte = typeSysObj.getElements();

		Iterator<SystemObject> iterator_Typen = typobjekte.iterator();

		while (iterator_Typen.hasNext()) {
			SystemObject sysObj = (SystemObject) iterator_Typen.next();

			objekte.add(sysObj);

		}

		return objekte;
	}

}
