/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.objektzusammenstellung;

import java.util.Date;
import java.util.LinkedList;

import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IViewReference;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

/**
 * Diese Klasse bietet Untersttzung beim ffnen der
 * SichtObjektzusammenstellung. Es knnen mehrere Instanzen der Sicht angelegt
 * werden, die SecondaryID zur eindeutigen Identifizierung wird zurckgeliefert.
 * 
 * @author HofmannJ
 * 
 */
public class OeffneSichtObjektzusammenstellung
{
	/**
	 * ID der zu ffnenden View
	 */
	private static final String _viewID = SichtObjektzusammenstellung.ID;

	/**
	 * Sucht eine Sicht vom Typ {@link SichtObjektzusammenstellung} und
	 * identifiziert diese eindeutig an Hand ihrer SecondaryID.
	 * 
	 * @param secId
	 *          SecondaryID der {@link SichtObjektzusammenstellung}, die gefunden
	 *          werden soll.
	 * @return Gefundene {@link SichtObjektzusammenstellung} oder
	 *         <code>null</code>, wenn diese
	 *         {@link SichtObjektzusammenstellung} nicht gefunden werden konnte.
	 */
	public static SichtObjektzusammenstellung sucheSicht(String secId)
	{
		SichtObjektzusammenstellung re = null;

		IViewReference reference = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage().findViewReference(_viewID,
						secId);
		IViewPart view = reference.getView(true);
		if (view instanceof SichtObjektzusammenstellung)
		{
			re = (SichtObjektzusammenstellung) view;
		}

		return re;
	}

	/**
	 * ffnet eine {@link SichtObjektzusammenstellung} an Hand ihrer SecondaryID.
	 * 
	 * @param secId
	 *          SecondaryID der {@link SichtObjektzusammenstellung}, die geffnet
	 *          werden soll.
	 * @return Liefert <code>true</code>, wenn die jeweilige
	 *         {@link SichtObjektzusammenstellung} geffnet werden konnte oder
	 *         <code>false</code>, wenn nicht (z.B. ist dies der Fall, wenn die
	 *         SecondaryID falsch ist bzw. nicht existiert).
	 */
	public static boolean oeffneSicht(String secId)
	{
		boolean gefunden = false;

		SichtObjektzusammenstellung sichtObjektzusammenstellung = sucheSicht(secId);
		if (sichtObjektzusammenstellung != null)
		{
			gefunden = true;
			IWorkbenchWindow window = PlatformUI.getWorkbench()
					.getActiveWorkbenchWindow();
			window.getActivePage().activate(sichtObjektzusammenstellung);
		}

		return gefunden;
	}

	/**
	 * Erzeugt eine neue {@link SichtObjektzusammenstellung} und liefert deren
	 * SecondaryID. Die SecondaryID wird an Hand der aktuellen Systemzeit
	 * bestimmt.
	 * 
	 * @return SecondaryID der erzeugten {@link SichtObjektzusammenstellung}.
	 */
	public static String erzeugeNeueSicht()
	{
		String secId = "";
		secId = secId + new Date().getTime();

		IWorkbenchWindow window = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow();

		try
		{
			window.getActivePage().showView(_viewID, secId,
					IWorkbenchPage.VIEW_VISIBLE);
			sucheSicht(secId).setSecondaryId(secId);
			oeffneSicht(secId);
		}
		catch (PartInitException e)
		{
			e.printStackTrace();
			secId = null;
		}

		return secId;
	}

	/**
	 * Liefert eine Liste der SecondaryIDs aller Sichten vom Typ
	 * {@link SichtObjektzusammenstellung}.
	 * 
	 * @return Liste aller SecondaryIDs
	 */
	public static LinkedList<String> sucheAlleSichtInstanzen()
	{
		LinkedList<String> viewSecIDs = new LinkedList<String>();

		IWorkbenchPage activePage = PlatformUI.getWorkbench()
				.getActiveWorkbenchWindow().getActivePage();
		System.out.println("ActivePage: " + activePage);

		if (activePage != null)
		{
			IViewReference[] viewReferences = activePage.getViewReferences();
			for (IViewReference viewReference : viewReferences)
			{
				if (viewReference.getId().equals(_viewID))
				{
					viewSecIDs.add(viewReference.getSecondaryId());
				}
			}
		}

		return viewSecIDs;
	}

}
