/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.basislib.ort;

import java.util.Iterator;
import java.util.LinkedList;

import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Betriebskilometer;
import de.bsvrz.buv.rw.basislib.ort.hilfsklassen.Strasse;
import de.bsvrz.dav.daf.main.ClientDavConnection;

/**
 * Filter, reprsentiert die Auswahl des Betriebskilometers.
 * 
 * @author HofmannJ
 * 
 */
public class FilterBetriebskilometer implements Cloneable
{
  /**
   * Verbindung zum DaV
   */
  private ClientDavConnection _connection = null;

  /**
   * Pid der selektierten Strae
   */
  private String _pidStrasse = null;

  /**
   * Selektierte {@link Richtung}
   */
  private Richtung _richtung = null;

  /**
   * Speichert den gewhlten Betriebskilometer
   */
  private long _betriebskilometerWert = -1;

  // /**
  // * TreeMap als "Referenz" auf ListenElementObjekte vom Typ typ.strae
  // */
  // private TreeMap<String, Object> _strassen = null;

  private LinkedList<String> _blocknummern = new LinkedList<String>();

  /**
   * Speichert die selektierte BlockNummer
   */
  private String _selektierteBlockNummer = null;

  /**
   * Speichert die bentigte Instanz der Klasse {@link Betriebskilometer}.
   */
  private Betriebskilometer _betriebsKilometer = null;

  /**
   * Konstruktor. bernimmt die Verbindung zum DaV.
   * 
   * @param connection
   *          Verbindung zum DaV.
   */
  public FilterBetriebskilometer(ClientDavConnection connection)
  {
    this._connection = connection;
    init();

  }

  /**
   * Fhrt die Initialisierung der bentigten Datenstrukturen durch.
   * 
   */
  private void init()
  {
    if (_connection != null)
    {
      _betriebsKilometer = new Betriebskilometer(_connection);
    }
  }

  /**
   * Liefert die (sortierte) Liste der Straen zurck.
   * 
   * @return Liste aller Straen oder <code>null</code>, wenn keine Liste der Straen bezogen werden konnte.
   */
  public LinkedList<Strasse> getStrassen()
  {
    if (_betriebsKilometer != null)
    {
      return _betriebsKilometer.getStrassen();
    }
    else
    {
      return null;
    }
  }

  /**
   * Setzt die Pid der selektierten Strae.
   * 
   * @param pid
   *          Pid der selektierten Strae
   */
  public void setSelektierteStrasse(String pid)
  {
    this._pidStrasse = pid;
  }

  /**
   * Liefert die PID der selektierten Strae.
   * 
   * @return PID der selektierten Strae
   */
  public String getPidSelektierteStrasse()
  {
    return _pidStrasse;
  }

  /**
   * Setzt die selektierte Richtung.
   * 
   * @param richtung
   *          Zu setzende Richtung
   */
  public void setSelektierteRichtung(Richtung richtung)
  {
    this._richtung = richtung;
  }

  /**
   * Liefert die selektierte Richtung.
   * 
   * @return Selektierte Richtung
   */
  public Richtung getSelektierteRichtung()
  {
    return _richtung;
  }

  /**
   * Setzt den Wert des Betriebskilometers. Gleichzeitig wird die Liste der verfgbaren Blocknummern aktualisiert. Sie
   * ist ber die Methode {@link FilterBetriebskilometer#getVerfuegbareBlockNummern()} verfgbar.
   * 
   * @param betriebskilometerWert
   *          Der zu setzende Wert des Betriebskilometers.
   */
  public void setBetriebskilometer(long betriebskilometerWert)
  {
    this._betriebskilometerWert = betriebskilometerWert;
  }

  /**
   * Liefert den gesetzten Betriebskilometer Wert.
   * 
   * @return Gesetzter Betriebskilometer Wert.
   */
  public long getBetriebskilometer()
  {
    return _betriebskilometerWert;
  }

  /**
   * Liefert die selektierte BlockNummer.
   * 
   * @return Selektierte BlockNummer
   */
  public String getSelektierteBlockNummern()
  {
    return _selektierteBlockNummer;
  }

  /**
   * Setzt die selektierte BlockNummer.
   * 
   * @param blocknummer
   *          Zu setzende, selektierte Blocknummer.
   */
  public void setSelektierteBlockNummer(String blocknummer)
  {
    _selektierteBlockNummer = blocknummer;
  }

  /**
   * Liefert eine Liste der zum gesetzten Betriebskilometer verfgbaren BlockNummern.<br>
   * Ist die Liste <code>null</code>, liegt der gesetzte Betriebskilometer <b>nicht</b> im LCL-Segment. <br>
   * Ist ein Eintrag <code>-1</code>, dann existiert zwar eine Betriebskilometereintrag, jedoch verfgt dieser ber
   * <b>keine</b> Blockzuordnung.<br>
   * 
   * @return Liste von BlockNummern.
   */
  public LinkedList<String> getVorhandeneBlockNummern()
  {

    LinkedList<String> vorhandeneBlockNummern = null;

    if ((_betriebsKilometer != null) && (_pidStrasse != null) && (_betriebskilometerWert > -1))
    {
      vorhandeneBlockNummern = _betriebsKilometer.pruefeBetriebsKilometerVorkommen(_pidStrasse, _betriebskilometerWert);

    }

    return vorhandeneBlockNummern;
  }

  public String toString()
  {
    String re = "===FilterBetriebskilometer - toString===\n";
    re = re + "PID Strae:         " + _pidStrasse + "\n";
    re = re + "Richtung:           " + _richtung + "\n";
    re = re + "Betriebskilometer:  " + _betriebskilometerWert + "\n";
    re = re + "Selektierter Block: " + _selektierteBlockNummer + "\n";
    return re;
  }

  /**
   * Enumerator fr die (Himmels-) Richtung
   * 
   * @author HofmannJ
   * 
   */
  public enum Richtung {
    /**
     * Nord
     */
    NORD,

    /**
     * Sd
     */
    SD,

    /**
     * Ost
     */
    OST,

    /**
     * West
     */
    WEST
  }


	public Object clone()
  {
  	FilterBetriebskilometer re = new FilterBetriebskilometer(_connection);
  	re._pidStrasse = _pidStrasse;
  	re._richtung = _richtung;
  	re._betriebskilometerWert = _betriebskilometerWert;  	
  	Iterator<String> it = _blocknummern.iterator();
  	while(it.hasNext())
  	{
  		re._blocknummern.add(it.next());
  	}  	
  	re._selektierteBlockNummer = _selektierteBlockNummer;
  	re._betriebsKilometer = _betriebsKilometer;  	
  	return re;
  }
}
