/*
 * This file is part of Rahmenwerk.
 *
 * Rahmenwerk is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License  as 
 * published by the Free Software Foundation, either version 3 of the
 * License.
 *
 * Rahmenwerk is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with Rahmenwerk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 * 
 *
 * Copyright (c) 2008 Dambach Werke GmbH.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 * Dambach Werke GmbH - initial API and implementation
 *
 *
 *
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */
package de.bsvrz.buv.rw.rw.dav.datenbasis.prelader;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import de.bsvrz.buv.rw.rw.Start;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindung;
import de.bsvrz.buv.rw.rw.dav.DaVVerbindungEreignis;
import de.bsvrz.buv.rw.rw.dav.IDaVVerbindungZuhoerer;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmDatenBasis;
import de.bsvrz.buv.rw.rw.dav.datenbasis.GtmListenElementObjekt;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.KmdParameter;
import de.bsvrz.buv.rw.rw.kommandozeilenparameter.Kommandozeilenparameter;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Vorladen aller SystemObjekte der DaV-Verbindung. Der Start der
 * Singleton-Instanz erfolgt durch die Klasse {@link Start#earlyStartup()}. <br>
 * Mit Hilfe eines {@link IDaVPreLader}-Listeners kann man sich darber
 * informieren lassen, ob die Daten vom DaV als Instanz der Klasse
 * {@link GtmDatenBasis} geladen wurden. Die Methode
 * {@link DaVPreLader#hinzufuegenListenerPreLader(IDaVPreLader)} fgt einen
 * solchen Listener hinzu und die Methode
 * {@link DaVPreLader#entfernenListenerPreLader(IDaVPreLader)} entfernt einen
 * solchen Listener. <br>
 * Zusammen mit einem {@link IDaVVerbindungZuhoerer}, der sich bei der Klasse
 * {@link DaVVerbindung} registrieren lsst, lsst sich explizit auf einen Abbau
 * der DaV-Verbindung reagieren. Erst wenn nach einem Verbindungsaufbau der
 * {@link IDaVPreLader} wieder grnes Licht gibt, sind die Daten der aktuellen
 * DaV-Verbindung verfgbar. <br>
 * Die Klasse ScihtObjektzusammenstellung aus der Basislib liefert ein schnes
 * Beispiel fr den kombinierten Einsatz von den Listenern {@link IDaVPreLader}
 * sowie {@link IDaVVerbindungZuhoerer}.<br>
 * Mit Hilfe der Methode {@link DaVPreLader#istInitialesLadenAbgeschlossen()}
 * kann geprft werden, ob ein initiales Laden der {@link GtmDatenBasis}
 * stattgefunden hat. Ist dies nicht der Fall empfiehlt es sich, einen
 * {@link IDaVPreLader}-Listener bei dieser Klasse anzumelden, bevor die
 * Methoden {@link DaVPreLader#getSystemObjekte()} und
 * {@link DaVPreLader#getGtmDatenBasis()} aufgerufen werden - existiert eine
 * Verbindung zum DaV, lsen diese das Laden der Datenbasis aus. Da dies ein
 * nebenlufiger Vorgang ist, liefern diese Methoden dann <code>null</code> und
 * nur angemeldete {@link IDaVPreLader}-Listener werden ber eine initale
 * Datenbasis informiert.
 * 
 * @author HofmannJ
 * 
 */
public class DaVPreLader
{
	/**
	 * Liste von Listenern, die ber den Status des {@link DaVPreLader} informiert
	 * werden mchten.
	 */
	private LinkedList<IDaVPreLader> _listener = new LinkedList<IDaVPreLader>();

	/**
	 * Speichert die {@link GtmDatenBasis}
	 */
	private GtmDatenBasis _gtmDatenBasis = null;

	/**
	 * Wird <code>true</code>, wenn das initiale Laden abgeschlossen ist. Wird
	 * <code>false</code>, wenn die DaV-Verbindung abgebaut wird.
	 */
	private boolean _initialesLadenAbgeschlossen = false;

	/**
	 * Dieser Listener reagiert auf den Auf- und Abbau der Verbindung zum DaV.
	 * Wird die DaV-Verbindung getrennt, so wird das Flag
	 * {@link DaVPreLader#_initialesLadenAbgeschlossen} wieder false. Wird die
	 * Verbindung zum DaV wieder aufgebaut, so wird das initiale Laden durch die
	 * Methode {@link DaVPreLader#systemObjekteInitialLaden()} angestoen, so dass
	 * nach einem Verbingungswiederaufbau die Daten des jeweiligen DaV zur
	 * Verfgung stehen.
	 */
	private IDaVVerbindungZuhoerer _verbindungZuhoerer = null;

	private static DaVPreLader _instanz = null;

	/**
	 * Privater Konstruktor
	 */
	private DaVPreLader()
	{
		if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
		{
			KmdParameter kommandoZeilenParameter = Kommandozeilenparameter
					.getInstanz().getKommandoZeilenParameter("-davSystemObjekteVorladen");

			if (kommandoZeilenParameter != null)
			{
				if (kommandoZeilenParameter.getWert().equals("true"))
				{
					systemObjekteInitialLaden();
				}
			}
		}

		_verbindungZuhoerer = new IDaVVerbindungZuhoerer()
		{

			@Override
			public void verbindungAbgebaut(DaVVerbindungEreignis e)
			{
				// System.out.println("DaVPreLader -> Verbindung Abgebaut");
				_initialesLadenAbgeschlossen = false;
			}

			@Override
			public void verbindungAufgebaut(DaVVerbindungEreignis e)
			{
				// System.out.println("DaVPreLader -> Verbindung Aufgebaut");
				_initialesLadenAbgeschlossen = false;
				systemObjekteInitialLaden();
			}

		};
		DaVVerbindung.getInstanz().hinzufuegenZuhoerer(_verbindungZuhoerer);

	}

	/**
	 * Liefert die Singleton-Instanz der Klasse {@link DaVPreLader}.
	 * 
	 * @return Singleton-Instanz der Klasse {@link DaVPreLader}
	 */
	public static DaVPreLader getInstanz()
	{
		// if (_instanz == null)
		// _instanz = new DaVPreLader();
		//        	  
		// return _instanz;

		if (_instanz == null)
		{
			synchronized (DaVPreLader.class)
			{
				if (_instanz == null)
					_instanz = new DaVPreLader();
			}
		}
		return _instanz;

		// return Innen.INSTANZ;
	}

	/**
	 * Fgt einen {@link IDaVPreLader}-Listener zur Liste der Listener, sofern
	 * dort noch nicht vorhanden, hinzu.<br>
	 * <b>Achtung:</b> Hinzugefgte {@link IDaVPreLader}-Listener
	 * <b><i>mssen</i></b> nach Ablauf des Lebenszyklus der jeweiligen Klasse
	 * wieder entfernt werden.
	 * 
	 * @param listener
	 *          Hinzuzufgender {@link IDaVPreLader}-Listener
	 */
	public void hinzufuegenListenerPreLader(IDaVPreLader listener)
	{
		synchronized (_listener)
		{
			if (!_listener.contains(listener))
			{
				_listener.add(listener);

				if (_gtmDatenBasis != null)
				{
					synchronized (_gtmDatenBasis)
					{
						if (_gtmDatenBasis.getSystemObjects().size() > 0)
						{
							listener.systemObjekteInitial(_gtmDatenBasis.flacheKopie());
						}
					}
				}

			}
		}
	}

	/**
	 * Entfernt einen {@link IDaVPreLader}-Listener von der Liste der Listener.
	 * 
	 * @param listener
	 *          Zu entfernender {@link IDaVPreLader}-Listener.
	 */
	public void entfernenListenerPreLader(IDaVPreLader listener)
	{
		synchronized (_listener)
		{
			if (_listener.contains(listener))
			{
				_listener.remove(listener);
			}
		}
	}

	/**
	 * Ldt die Daten aus dem DaV. Ob die Daten komplett & initialisierend geladen
	 * werden, oder ob nur ein Update durchgefhrt wird, wird durch den Enumerator
	 * {@link EnumLadeArt} definiert.
	 * 
	 * @param enumLadeArt
	 *          Definition der Ladeart.
	 */
	private void datenLaden(EnumLadeArt enumLadeArt)
	{
		final EnumLadeArt ladeArt = enumLadeArt;
		DaVVerbindung daVVerbindung = DaVVerbindung.getInstanz();

		if (daVVerbindung.istDaVVerbindungAktiv())
		{
			final ClientDavInterface connection = daVVerbindung.getDaVVerbindung();

			Job job = new Job("SystemObjekte vorladen")
			{

				@Override
				protected IStatus run(IProgressMonitor monitor)
				{

					monitor.beginTask("Lese SystemObjekte ein...", 1);

					try
					{

						if (ladeArt.equals(EnumLadeArt.INITIALESLADEN))
						{
							GtmDatenBasis gtmDatenBasis = new GtmDatenBasis(connection, true,
									monitor);
							_gtmDatenBasis = gtmDatenBasis;
							_initialesLadenAbgeschlossen = true;
							benachrichtigeListenerInitialisierung();
						} else if ((ladeArt.equals(EnumLadeArt.AKTUALISIERUNGSLADEN))
								&& (_gtmDatenBasis != null))
						{
							synchronized (_gtmDatenBasis)
							{
								_gtmDatenBasis.setAll();
							}
							benachrichtigeListenerAktualisierung();
						}

						monitor.worked(1);
					} finally
					{
						monitor.done();
					}

					return Status.OK_STATUS;
				}

			};

			job.schedule();

		}
	}

	/**
	 * Benachrichtigt alle registrieren {@link IDaVPreLader}-Listener darber,
	 * dass eine Aktualisierung der {@link GtmDatenBasis} statt gefunden hat.
	 */
	private void benachrichtigeListenerAktualisierung()
	{
		if (_gtmDatenBasis != null)
		{
			synchronized (_gtmDatenBasis)
			{
				if (_gtmDatenBasis.getSystemObjects().size() > 0)
				{
					synchronized (_listener)
					{
						if (_listener.size() > 0)
						{
							Iterator<IDaVPreLader> it = _listener.iterator();
							while (it.hasNext())
							{
								GtmDatenBasis flacheKopie = _gtmDatenBasis.flacheKopie();
								IDaVPreLader listener = it.next();
								listener.systemObjekteAktualisierung(flacheKopie);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Benachrichtigt alle registrierten {@link IDaVPreLader}-Listener darber,
	 * dass die Initialisierung der {@link GtmDatenBasis} statt gefunden hat.
	 */
	private void benachrichtigeListenerInitialisierung()
	{
		if (_gtmDatenBasis != null)
		{
			synchronized (_gtmDatenBasis)
			{
				if (_gtmDatenBasis.getSystemObjects().size() > 0)
				{
					synchronized (_listener)
					{
						if (_listener.size() > 0)
						{
							Iterator<IDaVPreLader> it = _listener.iterator();
							while (it.hasNext())
							{
								GtmDatenBasis flacheKopie = _gtmDatenBasis.flacheKopie();
								IDaVPreLader listener = it.next();
								listener.systemObjekteInitial(flacheKopie);
							}
						}
					}
				}
			}
		}
	}

	/**
	 * Lst das initiale Laden der {@link GtmDatenBasis} aus.
	 */
	public void systemObjekteInitialLaden()
	{
		datenLaden(EnumLadeArt.INITIALESLADEN);
	}

	/**
	 * Lst die Aktualisierung einer bereits vorhandenen {@link GtmDatenBasis}
	 * aus.
	 */
	public void systemObjekteAktualisierungsLaden()
	{
		datenLaden(EnumLadeArt.AKTUALISIERUNGSLADEN);
	}

	/**
	 * Prft, ob das initiale Laden abgeschlossen ist.
	 * 
	 * @return Liefert <code>true</code>, wenn bereits SystemObjkte geladen
	 *         wurden, sonst <code>false</code>.
	 */
	public boolean istInitialesLadenAbgeschlossen()
	{
		return _initialesLadenAbgeschlossen;
	}

	/**
	 * Liefert die Liste der bereits vorgeladenen {@link SystemObject}s. Es bietet
	 * sich an, vorher mit {@link DaVPreLader#istInitialesLadenAbgeschlossen()} zu
	 * prfen, ob ein initales Laden der Datenbasis durchgefhrt wurde. Ist dies
	 * nicht der Fall und ist eine Verbindung zum DaV aktiv, sollte ein
	 * {@link IDaVPreLader} bei dieser Klasse angemeldet werden. In diesem Fall
	 * lst diese Methode das Laden der Datenbasis aus und alle angemeldeten
	 * {@link IDaVPreLader} werden darber informiert, wenn eine Initiale
	 * Datenbasis verfgbar ist.
	 * 
	 * @return Liste der bereits vorgeladenen {@link SystemObject}s oder
	 *         <code>null</code>, wenn noch keine Systemobjekte vorgeladen sind.
	 */
	public LinkedList<SystemObject> getSystemObjekte()
	{
		// LinkedList<SystemObject> re = null;
		LinkedList<SystemObject> re = new LinkedList<SystemObject>();

		synchronized (this)
		{
			if (_gtmDatenBasis != null)
			{
				synchronized (_gtmDatenBasis)
				{
					Iterator<GtmListenElementObjekt> it = _gtmDatenBasis
							.getSystemObjects().iterator();
					while (it.hasNext())
					{
						GtmListenElementObjekt listenElementObjekt = it.next();
						SystemObject systemObjekt = listenElementObjekt.getSystemObjekt();
						if (systemObjekt != null)
						{
							re.add(systemObjekt);
						}
					}
				}
			} else if (!_initialesLadenAbgeschlossen)
			{
				if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
				{
					systemObjekteInitialLaden();
				}
			}
		}

		return re;
	}

	/**
	 * Liefert ein Kopie der {@link GtmDatenBasis}. Wird <code>null</code>
	 * zurckgeliefert, wurde noch keine Initiale {@link GtmDatenBasis} geladen.
	 * Es bietet sich an, vorher mit
	 * {@link DaVPreLader#istInitialesLadenAbgeschlossen()} zu prfen, ob ein
	 * initales Laden der Datenbasis durchgefhrt wurde. Ist dies nicht der Fall
	 * und ist eine Verbindung zum DaV aktiv, sollte ein {@link IDaVPreLader} bei
	 * dieser Klasse angemeldet werden. In diesem Fall lst diese Methode das
	 * Laden der Datenbasis aus und alle angemeldeten {@link IDaVPreLader} werden
	 * darber informiert, wenn eine Initiale Datenbasis verfgbar ist.
	 * 
	 * @return Kopie der {@link GtmDatenBasis} oder <code>null</code>, wenn noch
	 *         keine {@link GtmDatenBasis} vorgeladen werden konnte.
	 */
	public GtmDatenBasis getGtmDatenBasis()
	{
		GtmDatenBasis re = null;

		synchronized (this)
		{
			if (istInitialesLadenAbgeschlossen())
			{
				synchronized (_gtmDatenBasis)
				{
					re = _gtmDatenBasis.flacheKopie();
				}
			} else if (DaVVerbindung.getInstanz().istDaVVerbindungAktiv())
			{
				systemObjekteInitialLaden();
			}
		}

		return re;
	}

	/**
	 * Beschreibt die Art des Ladens: Initialisierung oder Aktualisierung.
	 * 
	 * @author HofmannJ
	 * 
	 */
	private enum EnumLadeArt
	{
		/**
		 * Initiales Laden der {@link GtmDatenBasis}.
		 */
		INITIALESLADEN,
		/**
		 * Aktualisierung einer vorhandenen {@link GtmDatenBasis}.
		 */
		AKTUALISIERUNGSLADEN;
	}
}
