/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2004 by Kappich+Kni Systemberatung, Aachen
 * 
 * This file is part of de.bsvrz.dav.dav.
 * 
 * de.bsvrz.dav.dav is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.dav.dav is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.dav.dav; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.dav.dav.main;

import de.bsvrz.dav.daf.main.ApplicationCloseActionHandler;
import de.bsvrz.sys.funclib.commandLineArgs.ArgumentList;
import de.bsvrz.sys.funclib.debug.Debug;

import java.io.BufferedReader;
import java.io.InputStreamReader;

/**
 * Klasse zum Start des Datenverteilers ohne grafische Oberflche.
 * 
 * @author Kappich Systemberatung
 * @version $Revision: 5963 $
 */
public class Transmitter {
	
	private static Debug _debug;
	
	public final static int _debugLevel = 0;
	
	private final ConnectionsManager _connectionsManager;
	
	public Transmitter(String[] args) throws Exception {
		ArgumentList arguments = new ArgumentList(args);
		Debug.init("Datenverteiler", arguments);
		_debug = Debug.getLogger();
		Thread.setDefaultUncaughtExceptionHandler(new UncaughtExceptionHandler());
		ServerDavParameters serverDavParameters = new ServerDavParameters(arguments);
		arguments.ensureAllArgumentsUsed();
		_debug.info("Datenverteiler wird gestartet");
		_connectionsManager = new ConnectionsManager(serverDavParameters);
		_debug.info("Datenverteiler bereit");
		if(_debugLevel > 0) {
			BufferedReader in = new BufferedReader(new InputStreamReader(System.in));
			String inputLine;
			while(true) {
				inputLine = in.readLine();
				if(inputLine == null) {
					// nicht mehr weiter lesen, aber nicht das Programm terminieren
					break;
				}
				else if(inputLine.startsWith("q")) {
					//Programm kontrolliert terminieren
					System.exit(0);
				}
				else if(inputLine.startsWith("s")) {
					_connectionsManager.printSubscriptions();
				}
			}
		}
	}
	
	ConnectionsManager getConnectionsManager() {
		return _connectionsManager;
	}
	
	/**
	 * Setzt den {@link de.bsvrz.dav.daf.main.ApplicationCloseActionHandler}.
	 * 
	 * @param closer
	 *            der CloseActionHandler
	 */
	public void setCloseHandler(ApplicationCloseActionHandler closer) {
		getConnectionsManager().selfClientDavConnection.setCloseHandler(closer);
	}
	
	/**
	 * Start-Funktion des Datenverteilers.
	 * 
	 * @param arguments
	 *            Aufrufargumente
	 */
	public static void main(String[] arguments) {
		try {
			new Transmitter(arguments);
		}
		catch(Exception e) {
			if(_debug == null) _debug = Debug.getLogger();
			_debug.error("Fehler im Datenverteiler", e);
			System.exit(1);
		}
	}

	/**
	 * Implementierung eines UncaughtExceptionHandlers, der bei nicht abgefangenen Exceptions und Errors entsprechende Ausgaben macht und im Falle eines Errors den
	 * Prozess terminiert.
	 */
	private static class UncaughtExceptionHandler implements Thread.UncaughtExceptionHandler {

		/** Speicherreserve, die freigegeben wird, wenn ein Error auftritt, damit die Ausgaben nach einem OutOfMemoryError funktionieren */
		private volatile byte[] _reserve = new byte[20000];

		public void uncaughtException(Thread t, Throwable e) {
			if(e instanceof Error) {
				// Speicherreserve freigeben, damit die Ausgaben nach einem OutOfMemoryError funktionieren
				_reserve = null;
				try {
					System.err.println("Schwerwiegender Laufzeitfehler: Ein Thread hat sich wegen eines Errors beendet, Prozess wird terminiert");
					System.err.println(t);
					e.printStackTrace(System.err);
					_debug.error("Schwerwiegender Laufzeitfehler: " + t + " hat sich wegen eines Errors beendet, Prozess wird terminiert", e);
				}
				catch(Throwable ignored) {
					// Weitere Fehler whrend der Ausgaben werden ignoriert, damit folgendes exit() auf jeden Fall ausgefhrt wird.
				}
				System.exit(1);
			}
			else {
				System.err.println("Laufzeitfehler: Ein Thread hat sich wegen einer Exception beendet:");
				System.err.println(t);
				e.printStackTrace(System.err);
				_debug.warning("Laufzeitfehler: " + t + " hat sich wegen einer Exception beendet", e);
			}
		}
	}

}
