/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2004 by Kappich+Kni Systemberatung Aachen (K2S)
 * 
 * This file is part of de.bsvrz.pat.sysbed.
 * 
 * de.bsvrz.pat.sysbed is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.pat.sysbed is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.pat.sysbed; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.pat.sysbed.dataview;

import de.bsvrz.dav.daf.main.config.Attribute;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.sys.funclib.debug.Debug;

import javax.swing.*;
import javax.swing.border.EtchedBorder;
import java.awt.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.util.*;
import java.util.List;

/**
 * Die Klasse <code>HeaderGrid</code> dient zur Darstellung einer hierarchischen Struktur (z.B. Baum). Die Darstellung kann dazu verwendet werden, einen
 * verschachtelten Tabellenkopf einer Tabelle zu erstellen. Ein Objekt dieser Klasse besteht aus einem Vater-Element und beliebig vielen Nachfolgern. Bei der
 * Darstellung werden die Nachfolger mittels eines {@link HeaderGrid.Splitter Schiebereglers} voneinander getrennt.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5052 $
 */
public class HeaderGrid {

	/** der Debug-Logger */
	private Debug _debug = Debug.getLogger();

	/* ############# Variablen ############## */
	/** speichert das Objekt des Schiebereglers */
	private final Splitter _splitter;

	/** speichert die Attributliste oder das Attribut, welches dargestellt werden soll */
	private final HeaderElement _headerElement;

	/** Liste aller Nachfolger vom Vater-Element */
	private final List<HeaderGrid> _successors;

	/** speichert alle angemeldeten Objekte, u.a. zwecks Grenanpassung */
	private final List<RowListener> _rowListener;

	/** speichert ein Exemplar des DataViewPanels */
	private final DataViewPanel _dataViewPanel;

	/** Gibt an, ob die mouseDragged-Methode des Splitters von einem Vaterknoten aus aufgerufen wurde. */
	private boolean _fromParent = false;

	/** Gibt an, ob die Spalte von bergeordneten Spalte geschlossen wurde oder nicht. */
	private boolean _closedFromAbove = true;

	/** Gibt die prozentuale Vernderung des Vater-Elements beim Vergrern bzw. Verkleinern an. */
	private int _percent = 0;

	/** Speichert die Spaltenbreite, bevor die Spalte verkleinert wird. */
	private int _lastColumnWidth = 0;

	/** Speichert zu jedem Element die Anzahl der Spalten unter sich. */
	private int _numberOfColumns = 0;

	/** Speichert den Vaterknoten. */
	private HeaderGrid _parent = null;

	/* ########### Konstruktor ############ */

	/**
	 * Es wird ein Objekt der Klasse <code>HeaderGrid</code> erzeugt.
	 *
	 * @param parent        Vater-Knoten, dieses Knotens
	 * @param node          eine Attributgruppe oder ein Attribut
	 * @param dataViewPanel Komponente, in der die Datenstze dargestellt werden
	 */
	public HeaderGrid(HeaderGrid parent, Object node, DataViewPanel dataViewPanel) {
		_parent = parent;
		_dataViewPanel = dataViewPanel;
		_splitter = new Splitter();
		_headerElement = new HeaderElement(node);
		_successors = new ArrayList<HeaderGrid>();
		_rowListener = new ArrayList<RowListener>();
	}


	/* #### Zeilenobjekte melden sich beim Header an, damit u.a. Grennderung weitergereicht werden knnen ### */
	/**
	 * Objekte, die sich hier anmelden, werden benachrichtigt, sobald im Spaltenheader ein Schieberegler bewegt wird.
	 *
	 * @param rowListener anzumeldendes Objekt
	 */
	public void addRowListener(RowListener rowListener) {
		_rowListener.add(rowListener);
	}

	/**
	 * Objekt, welches beim Listener wieder abgemeldet werden soll.
	 *
	 * @param rowListener abzumeldendes Objekt
	 */
	public void removeRowListener(RowListener rowListener) {
		_rowListener.remove(rowListener);
	}

	/**
	 * An alle angemeldeten Objekte wird die neue Breite der vernderten Spalte bergeben.
	 *
	 * @param width neue Spaltenbreite
	 */
	public void setRowWidth(int width) {
		for(Iterator iterator = _rowListener.iterator(); iterator.hasNext();) {
			RowListener rowListener = (RowListener)iterator.next();
			rowListener.setWidth(width);
		}
	}

	/**
	 * Liefert die optimale Spaltenbreite, so dass alle Eintrge zu sehen sind.
	 *
	 * @return optimale Spaltenbreite
	 */
	public int getOptimalColumnWidth() {
		int optimalColumnWidth = 0;
		for(Iterator iterator = _rowListener.iterator(); iterator.hasNext();) {
			RowListener rowListener = (RowListener)iterator.next();
			int width = rowListener.getOptimalColumnWidth();
			if(width > optimalColumnWidth) optimalColumnWidth = width;
		}
		return optimalColumnWidth;
	}


	/* ############# Get-/ Set- und Add-Methoden ############# */
	/**
	 * Fgt einen Nachfolger hinzu.
	 *
	 * @param headerGrid ein Nachfolger
	 */
	public void addHeaderSuccessor(HeaderGrid headerGrid) {
		_successors.add(headerGrid);
	}

	/**
	 * Gibt die Nachfolger des Grids zurck.
	 *
	 * @return die Nachfolger
	 */
	public List<HeaderGrid> getHeaderSuccessors() {
		return _successors;
	}

	/**
	 * Gibt das Vater-Element zurck.
	 *
	 * @return Vater-Element
	 */
	public HeaderElement getHeaderElement() {
		return _headerElement;
	}

	/**
	 * Gibt den Schieberegler zurck.
	 *
	 * @return Schieberegler
	 */
	public Splitter getSplitter() {
		return _splitter;
	}

	/**
	 * Gibt die Anzahl der Spalten zurck, die sich unterhalb dieses Grids befinden.
	 *
	 * @return Anzahl der Spalten
	 */
	public int getNumberOfColumns() {
		return _numberOfColumns;
	}

	/**
	 * Setzt die Anzahl der Spalten, die sich unterhalb dieses Grids befinden.
	 *
	 * @param numberOfColumns Anzahl der Spalten
	 */
	public void setNumberOfColumns(int numberOfColumns) {
		_numberOfColumns = numberOfColumns;
	}


	/* ########### Konstruktion des Headers ############# */
	/**
	 * Erstellt ein Panel aus den im Objekt gesammelten Daten. Dieses Objekt besteht aus einem Element und seinen Nachfolgern. Wird fr den Aufbau eines
	 * hierarchischen Tabellenkopfes genutzt.
	 *
	 * @return das Panel des Objekts
	 */
	public JPanel createHeader() {
		int column = 0;
		int row = 0;

		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel panel = new JPanel(gbl);
		// es gibt genau einen Header (nicht mehr und nicht weniger)
		int number = _successors.size();
		if(number == 0) number = 1;
		double weighty = 0;            // die Y-Ausdehnung des Headers in Abhngigkeit, ob er Nachfolger hat oder nicht
		if(_successors.size() == 0) {
			weighty = 100;
		}
		gbc = makeGBC(column, row++, number, 1, 100, weighty);
		gbc.fill = GridBagConstraints.BOTH;
		JPanel headerPanel = createHeaderElement();
		gbl.setConstraints(headerPanel, gbc);
		panel.add(headerPanel);

		// Nachfolger ...
		for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
			HeaderGrid headerGrid = (HeaderGrid)iterator.next();
			gbc = makeGBC(column++, row, 1, 1, 100, 100);
			gbc.fill = GridBagConstraints.BOTH;

			JPanel gridPanel = headerGrid.createHeader();
			gbl.setConstraints(gridPanel, gbc);
			panel.add(gridPanel);
		}
		return panel;
	}

	/**
	 * Erstellt ein Panel, welches ein Element mit zugehrigem Schieberegler auf der rechten Seite darstellt.
	 *
	 * @return das Panel des Objekts
	 */
	private JPanel createHeaderElement() {
		GridBagLayout gbl = new GridBagLayout();
		GridBagConstraints gbc = new GridBagConstraints();
		JPanel panel = new JPanel();
		panel.setLayout(gbl);
		gbc = makeGBC(0, 0, 1, 1, 100, 100);
		gbc.fill = GridBagConstraints.BOTH;
		gbl.setConstraints(_headerElement, gbc);
		panel.add(_headerElement);

		// Splitter einfgen
		gbc = makeGBC(1, 0, 1, 1, 0, 0);
		gbc.fill = GridBagConstraints.VERTICAL;
		gbl.setConstraints(_splitter, gbc);
		panel.add(_splitter);

		return panel;
	}

	/**
	 * Hilfsfunktion zur Konstruktion des Panels. Hierber werden die Bedingungen fr die Anordnung der Elemente gesetzt.
	 *
	 * @param gridx      Spaltennummer
	 * @param gridy      Zeilennummer
	 * @param gridwidth  Anzahl der Spalten ber die das Element reicht
	 * @param gridheight Anzahl der Zeilen ber die das Element reicht
	 * @param weightx    Verteilung von zur Verfgung stehendem Platz (horizontal)
	 * @param weighty    Verteilung von zur Verfgung stehendem Platz (vertikal)
	 *
	 * @return die Bedingungen fr die Anordnung des Elements
	 */
	private GridBagConstraints makeGBC(int gridx, int gridy, int gridwidth, int gridheight, double weightx, double weighty) {
		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = gridx;
		gbc.gridy = gridy;
		gbc.gridwidth = gridwidth;
		gbc.gridheight = gridheight;
		gbc.weightx = weightx;
		gbc.weighty = weighty;
		return gbc;
	}


	

	/**
	 * Setzt bei allen Komponenten den Cursor, sobald der Schieberegler bettigt wird.
	 *
	 * @param grid   das Grid, welches den Cursor zugewiesen bekommt
	 * @param cursor den Wert fr den Cursor
	 *
	 * @see Cursor
	 * @see Component#setCursor(java.awt.Cursor)
	 */
	private void setHeaderCursors(HeaderGrid grid, int cursor) {
		grid.getHeaderElement().setCursor(new Cursor(cursor));
		// alle Nachfolger
		List successors = grid.getHeaderSuccessors();
		for(Iterator iterator = successors.iterator(); iterator.hasNext();) {
			HeaderGrid headerGrid = (HeaderGrid)iterator.next();
			setHeaderCursors(headerGrid, cursor);
		}
	}

	// Pfeil selbst malen
//    private Arrow getLeftArrow() {
//        if (_leftArrow == null) {
//            _leftArrow = new Arrow(SwingConstants.WEST);
//        }
//        return _leftArrow;
//    }

//    private Arrow getRightArrow() {
//        if (_rightArrow == null) {
//            _rightArrow = new Arrow(SwingConstants.EAST);
//        }
//        return _rightArrow;
//    }

	/* ############## Innere Klasse Splitter von der Klasse HeaderGrid ############## */

	/**
	 * Die Klasse implementiert einen Schieberegler. Mit diesem Regler wird eine links benachbarte Spalte durch Benutzung der Maus vergrert bzw. verkleinert,
	 * indem der Schieberegler nach links bzw. rechts verschoben wird.
	 *
	 * @see HeaderElement
	 * @see RowListener
	 */
	class Splitter extends JPanel implements MouseListener, MouseMotionListener {

		/* ######### Variablen ######### */
		/** speichert die Breite des Schiebereglers */
		private final int _splitterWidth = 5;

		/** speichert bei bettigen der Maustaste die aktuelle Mausposition innerhalb des Schiebereglers */
		private int _posInSplitter;

		/** speichert den nach rechts gerichteten Pfeil incl. seiner Funktionalitt */
		private Arrow _icon = null;


		/* ######### Konstruktor ########## */
		/** Erzeugt ein Objekt der Klasse <code>Splitter</code>. */
		public Splitter() {
			// Mauscursor setzen
			setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
			


			setBorder(new EtchedBorder());
//            setBackground(Color.gray);
			setPreferredSize(new Dimension(_splitterWidth, 1));

			addMouseListener(this);
			addMouseMotionListener(this);
		}


		/* ############# Public-Methoden ############## */
		/**
		 * Setzt die Variable im gesamten untergeordneten Grid, welche angibt, ob die mouseDragged-Methode des Schiebereglers von einem Vaterknoten aus aufgerufen
		 * wurde.
		 *
		 * @param grid       wo die Variable gesetzt werden soll
		 * @param fromParent neuer Wert der Variable
		 */
		private void setFromParent(HeaderGrid grid, boolean fromParent) {
			for(Iterator iterator = grid.getHeaderSuccessors().iterator(); iterator.hasNext();) {
				HeaderGrid headerGrid = (HeaderGrid)iterator.next();
				headerGrid._fromParent = false;
				headerGrid.getSplitter().setFromParent(headerGrid, fromParent);
			}
		}

		/** Fgt den Pfeil zum Aufklappen in den Schieberegler ein. */
		public void addRightArrow() {
			if(_icon == null) {
//                _icon = new JLabel(getHeaderElement()._rightArrow);
				_icon = new Arrow(SwingConstants.EAST);
				_icon.addMouseListener(
						new MouseAdapter() {
							/**
							 * Die ursprngliche Breite wird wieder hergestellt und die Pfeile in den betroffenen Schiebereglern werden entfernt.
							 *
							 * @param e Maus-Ereignis
							 */
							public void mouseClicked(MouseEvent e) {
								_closedFromAbove = true;
								undoColumnWidth();
								removeUpperArrows();
								removeLowerArrows();
							}

							/**
							 * Falls die Spalte durch ziehen wieder verbreitert wurde, dann werden die Pfeile in den betroffenen Schiebereglern entfernt.
							 *
							 * @param e Maus-Ereignis
							 */
							public void mouseReleased(MouseEvent e) {
								int elementWidth = getHeaderElement().getSize().width;
								if(elementWidth > ((_numberOfColumns - 1) * _splitterWidth)) {
									_fromParent = false;
									_closedFromAbove = true;
									removeLowerArrows();
									removeUpperArrows();
									setFromParent(_dataViewPanel.getHeaderGrid(), false);
								}
							}
						}
				);
				_icon.addMouseMotionListener(_splitter);
			}
			setBorder(BorderFactory.createEmptyBorder());
			setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			setLayout(new BorderLayout());
			add(_icon, BorderLayout.CENTER);
		}

		/** Setzt die optimale Spaltenbreite, so dass alle Daten vollstndig zu sehen sind. */
		public void setOptimalColumnWidth() {
			if(_successors.isEmpty()) {    // Blattknoten
				if(_closedFromAbove) {
					int width = getOptimalColumnWidth();
					width += 10; // ein klein wenig Rand um die Komponente freilassen
					setColumnWidth(width);
				}
			}
			else {    // Nachfolger bestimmen
				for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
					HeaderGrid headerGrid = (HeaderGrid)iterator.next();
					headerGrid.getSplitter().setOptimalColumnWidth();
				}
			}
		}


		/* ######### MouseListener und MouseMotionListener - Methoden ############ */
		/** Beim Doppelklick wird die optimale Spaltenbreite in Abhngigkeit der angezeigten Daten ermittelt und gesetzt. */
		public void mouseClicked(MouseEvent e) {
			if(e.getClickCount() > 1) {         // Doppelclick
				setOptimalColumnWidth();
				return;
			}
		}

		/** wird hier nicht bentigt */
		public void mouseEntered(MouseEvent e) {
		}

		/** wird hier nicht bentigt */
		public void mouseExited(MouseEvent e) {
		}

		/** wird hier nicht bentigt */
		public void mouseMoved(MouseEvent e) {
		}

		/**
		 * Beim Bettigen der linken Maustaste wird die aktuelle Position der Maus innerhalb des Schiebereglers abgespeichert. Ist dies der erste Aufruf, dann werden
		 * die Gren der Elemente mittels der Methode {@link DataViewPanel#setHeaderSizes(HeaderGrid,int)} initialisiert.
		 *
		 * @param e Mausereignis
		 */
		public void mousePressed(MouseEvent e) {
			if(_dataViewPanel != null) {
				if(_dataViewPanel.getFirstRun()) {
//					System.out.println("Mouse");
					_dataViewPanel.initHeaderSize();
					_dataViewPanel.setFirstRun(false);
				}
			}

			// Position im Splitter
			_posInSplitter = e.getX();

			// Aussehen des Cursors setzen
			setHeaderCursors(_dataViewPanel.getHeaderGrid(), Cursor.E_RESIZE_CURSOR);
			_dataViewPanel.setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));

			// ursprngliche Breite merken
			setLastColumnWidth();
		}

		/** Speichert in allen untergeordneten Spalten, die vorher noch nicht geschlossen wurden, die Spaltenbreite. */
		private void setLastColumnWidth() {
			if(_closedFromAbove) { // nur hier die Variable _lastColumnWidth ndern
				if(_successors.isEmpty()) {
					int elementWidth = getHeaderElement().getSize().width;  // aktuelle Breite merken
					_lastColumnWidth = elementWidth + _splitterWidth;
				}
				else {
					for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
						HeaderGrid headerGrid = (HeaderGrid)iterator.next();
						headerGrid.getSplitter().setLastColumnWidth();
					}
				}
			}
		}

		/**
		 * Beim Loslassen der linken Maustaste werden ggf. gesetzte Werte zurckgesetzt.
		 *
		 * @param e Mausereignis
		 */
		public void mouseReleased(MouseEvent e) {
			int elementWidth = getHeaderElement().getSize().width;
			if(elementWidth == (_numberOfColumns - 1) * _splitterWidth) {
				getHeaderElement().addUpperArrows();
			}
			addRightArrows();

			setHeaderCursors(_dataViewPanel.getHeaderGrid(), Cursor.DEFAULT_CURSOR);
			_dataViewPanel.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

		/** Fgt bei darunter liegenden Elementen einen Pfeil ein, falls noch keiner vorhanden ist. */
		private void addRightArrows() {
			_fromParent = false;
			int elementWidth = getHeaderElement().getSize().width;
			if(elementWidth == (_numberOfColumns - 1) * _splitterWidth) {
				addRightArrow();
			}
			for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
				HeaderGrid headerGrid = (HeaderGrid)iterator.next();
				headerGrid.getSplitter().addRightArrows();
			}
		}

		/**
		 * Bei gedrckter linker Maustaste und ziehen der Maus, wird links stehende Spalte vergrert bzw. verkleinert. Der Layoutmanager macht sein briges und passt
		 * alle anderen Elemente ggf. an.
		 *
		 * @param e Mausereignis
		 */
		public void mouseDragged(MouseEvent e) {
			if(_successors.size() > 0) {   // Nachfolger? -> Nachfolger benachrichtigen
				if(!_fromParent) {         // Methode wurde direkt aufgerufen
					// prozentuale Vernderung ermitteln
					Dimension size = _headerElement.getSize();
					int elementX = size.width;
					int deltaX = e.getX() - _posInSplitter;
					if(elementX > 0) {
						_percent = (int)((deltaX * 100) / elementX);
					}
					else {
						_percent = 0;
					}
					// falls die Gre des Elements gleich null ist, es aber vergrert werden soll
					if(deltaX > 0 && _percent == 0) {
						_percent = 1;
					}
				}
				// prozentuale Vernderung an Nachfolger weiterleiten
				for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
					HeaderGrid headerGrid = (HeaderGrid)iterator.next();
//                    if (headerGrid._closedFromAbove) {  // sonst braucht der Zweig nicht weiter verfolgt werden
					headerGrid._percent = _percent;
					headerGrid._fromParent = true;
					headerGrid._splitter.mouseDragged(e);
//                    }
				}
			}
			else {                // keine Nachfolger -> Blattelement, Gre anpassen
				// neue Breite ermitteln und anwenden
				Dimension size = _headerElement.getSize();
				int elementX = size.width;
				int elementY = size.height;
				if(_fromParent) {
					// Methode wurde von einem bergeordneten Element aufgerufen.
					if(_closedFromAbove) {     // falls closedFromAbove == false -> muss Element geschlossen bleiben
						if(_percent > 0 && elementX == 0) {
							elementX = 1;
						}
						elementX += (int)((elementX * _percent) / 100);
						if(elementX <= 0) elementX = 0;
					}
				}
				else {
					// Methode wurde direkt aufgerufen
					elementX += e.getX() - _posInSplitter;
					if(elementX <= 0) elementX = 0;
				}
				size = new Dimension(elementX, elementY);
				_headerElement.setPreferredSize(size);
				_headerElement.revalidate();
				setRowWidth(size.width + _splitterWidth);
			}
		}


		/* ############ Private-Methoden ############# */
		/**
		 * Setzt die Spaltenbreite auf einen beliebigen Wert.
		 *
		 * @param width die neue Spaltenbreite
		 */
		private void setColumnWidth(int width) {
			_headerElement.setPreferredSize(new Dimension(width - _splitterWidth, _headerElement.getPreferredSize().height));
			_headerElement.revalidate();
			setRowWidth(width);
		}

		/** Entfernt den Pfeil zum Aufklappen aus dem Schieberegler. */
		private void removeRightArrow() {
			remove(_icon);
			setBorder(new EtchedBorder());
			setCursor(new Cursor(Cursor.E_RESIZE_CURSOR));
		}

		/** Alle darber liegenden Elemente werden berprft, ob der Pfeil aus dem Schieberegler entfernt werden kann. Falls ja, dann wird er entfernt. */
		private void removeUpperArrows() {
			if(_parent != null) {      // ist parent == null -> fertig
				int count = _parent.getSplitter().getComponentCount();
				if(count > 0) {    // hat Parent einen Pfeil? -> entfernen und weiter nach oben gehen, sonst fertig
					_parent.getSplitter().removeRightArrow();
					_parent._closedFromAbove = true;
					_parent.getSplitter().removeUpperArrows();  // einen hher
				}
			}
		}

		/** Bei allen darunter liegenden Elementen werden die Pfeile aus den Schiebereglern entfernt. */
		private void removeLowerArrows() {
			if(_closedFromAbove) {
				removeRightArrow();
				for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
					HeaderGrid headerGrid = (HeaderGrid)iterator.next();
					headerGrid.getSplitter().removeLowerArrows();
				}
			}
		}

		/** Die Spaltenbreite wird wieder auf den ursprnglichen Wert gesetzt. */
		private void undoColumnWidth() {
			if(_closedFromAbove) {
				if(_successors.isEmpty()) {
					setColumnWidth(_lastColumnWidth);
				}
				else {
					for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
						HeaderGrid headerGrid = (HeaderGrid)iterator.next();
						headerGrid.getSplitter().undoColumnWidth();
					}
				}
			}
		}
	}


	/* ############# Diese Klasse reprsentiert ein Attribut ############## */
	/** Diese Klasse reprsentiert eine Zelle des hierarchischen {@link HeaderGrid Spaltenheaders}. */
	public class HeaderElement extends JComponent {

		/* ########### Variablen ############ */
		/** speichert das Objekt, welches im Header dargestellt werden soll */
		private final Object _headerObject;

		/** speichert den darzustellenden Text */
		private final String _text;

		/** speichert die Pid der Attributgruppe bzw. des Attributes */
		private final String _pid;


		/* ############ Konstruktor ################ */
		/**
		 * Konstruktor. Zeichnet ein HeaderElement.
		 *
		 * @param headerElement vom Typ <code>AttributeGroup</code> oder <code>Attribute</code>
		 */
		public HeaderElement(Object headerElement) {
			_headerObject = headerElement;
			if(_headerObject instanceof Attribute) {
				Attribute attribute = (Attribute)_headerObject;
				if(isAttributeArray(attribute)) {
					_text = attribute.getNameOrPidOrId() + " [..]";
				}
				else {
					_text = attribute.getNameOrPidOrId();
				}
				_pid = attribute.getPid();
			}
			else if(_headerObject instanceof AttributeGroup) {
				AttributeGroup attributeGroup = (AttributeGroup)_headerObject;
				_text = attributeGroup.getNameOrPidOrId();
				_pid = attributeGroup.getPid();
			}
			else {
				_debug.error("Unerlaubtes Objekt - muss Attributgruppe oder Attribut sein!");
				_text = "";
				_pid = null;
			}
			createHeaderElement();
		}

		/** Diese Methode ist fr das Aussehen und die Funktionalitt des HeaderElements verantwortlich. */
		private void createHeaderElement() {
			JLabel label = new JLabel(_text);
			label.setHorizontalAlignment(SwingConstants.CENTER);

			Arrow icon = new Arrow(SwingConstants.WEST);
			icon.addMouseListener(
					new MouseAdapter() {
						public void mouseClicked(MouseEvent e) {
							addLowerArrows();
							_closedFromAbove = false;       // den, den ich direkt schliee auf false setzen
							addUpperArrows();
						}
					}
			);

			




			setBorder(new EtchedBorder());
			setLayout(new BorderLayout());
			add(label, BorderLayout.CENTER);
			add(icon, BorderLayout.EAST);
		}


		/* ############# Hilfsmethoden ############ */
		/** Fgt bei hierarchisch hher liegenden Elementen Pfeile hinzu, wenn sie auch komplett geschlossen sind. */
		private void addUpperArrows() {
			if(_parent != null) {
				int parentWidth = _parent.getHeaderElement().getSize().width;
				int closedColumnsWidth = (_parent.getNumberOfColumns() - 1) * getSplitter()._splitterWidth;
				int elementWidth = getHeaderElement().getSize().width;
				if((parentWidth - closedColumnsWidth) == elementWidth || parentWidth == elementWidth) {
					// Pfeil hinzufgen
					_parent.getSplitter().addRightArrow();
					_parent._closedFromAbove = false;
					_closedFromAbove = true;
					_parent.getHeaderElement().addUpperArrows();
				}
			}
		}

		/**
		 * Beim Bettigen des Pfeils zum Einklappen der Spalte, wird dieser Spalte und allen darunterliegenden der Pfeil zum Aufklappen zur Verfgung gestellt, sowie
		 * die Spalte reduziert auf ein Minimum.
		 */
		private void addLowerArrows() {
			if(_closedFromAbove) {
				getSplitter().addRightArrow();
				if(_successors.isEmpty()) {
					_lastColumnWidth = getSize().width + getSplitter().getWidth();  // alte Breite merken
					getSplitter().setColumnWidth(getSplitter().getWidth()); // auf Splitter-Gre setzen
				}
				else {
					for(Iterator iterator = _successors.iterator(); iterator.hasNext();) {
						HeaderGrid headerGrid = (HeaderGrid)iterator.next();
						headerGrid.getHeaderElement().addLowerArrows(); // alle untergeordneten Elemente bedenken
					}
				}
			}
		}

		/**
		 * Prft, ob es sich bei dem Attribut um ein Array handelt, oder nicht.
		 *
		 * @param attribute das zu prfende Attribut
		 *
		 * @return ist das Attribut ein Array oder nicht
		 */
		private boolean isAttributeArray(Attribute attribute) {
			return attribute.isArray();
		}


		/* ########### Get-Methoden ############ */
		/**
		 * Gibt das Objekt des HeaderElements zurck.
		 *
		 * @return das Objekt des HeaderElements
		 */
		public Object getObject() {
			return _headerObject;
		}

		/**
		 * Gibt den Text des HeaderElements zurck, welcher dargestellt wird.
		 *
		 * @return den Text
		 */
		public String getText() {
			return _text;
		}

		/**
		 * Gibt die Pid des HeaderElements zurck. Besitzt es keine Pid wird null zurckgegeben.
		 *
		 * @return die Pid
		 */
		public String getPid() {
			if(_pid == null || _pid.equals("")) {
				return _text;
			}
			else {
				return _pid;
			}
		}
	}


	/* ################ Klasse Arrow ############# */
	/**
	 * Erstellt einen Pfeil in einer angegebenen Richtung und Farbe der Gre 5 x 5 Pixel. Die Richtung wird mit den Swing-Konstanten angegeben:<p>
	 * SwingConstants.WEST<br> SwingConstants.EAST
	 */
	class Arrow extends JPanel implements SwingConstants {

		/* ############ Variablen ############# */
		/** Richtung des Pfeils */
		private final int _direction;

		/** Farbe des Pfeils */
		private final Color _arrowColor;


		/* ############## Konstruktoren ############### */
		/**
		 * Erstellt einen Pfeil mit der angegebenen Richtung in der Farbe grau.
		 *
		 * @param direction von SwingConstants (WEST, EAST)
		 */
		public Arrow(int direction) {
			this(direction, Color.gray);
		}

		/**
		 * Erstellt einen Pfeil mit der angegebenen Richtung und der angegebenen Farbe.
		 *
		 * @param direction  von SwingConstants (WEST, EAST)
		 * @param arrowColor eine beliebige Farbe
		 */
		public Arrow(int direction, Color arrowColor) {
			_direction = direction;
			_arrowColor = arrowColor;
		}


		/* ################### Public-Methoden ################# */
		/**
		 * Zeichnet ein Dreieck auf das Panel.
		 *
		 * @param g Grafik-Komponente
		 */
		public void paint(Graphics g) {
			int w, h;

			w = getSize().width;
			h = getSize().height;

			g.setColor(getBackground());
			g.fillRect(0, 0, w, h);     // ohne Rand

			int mid = (h / 2);
			// Dreieck zeichnen
			g.setColor(_arrowColor);
			switch(_direction) {
				case WEST:
					for(int i = 0; i < 5; i++) {
						g.drawLine(i, mid - i, i, mid + i);
					}
					break;
				case EAST:
					for(int i = 0, j = 4; i < 5; i++, j--) {
						g.drawLine(j, mid - i, j, mid + i);
					}
					break;
			}
		}

		/**
		 * Gibt eine feste Gre fr den Pfeil zurck.
		 *
		 * @return festen Wert in Pfeilgre
		 */
		public Dimension getPreferredSize() {
			return new Dimension(getSplitter().getWidth() + 1, 5);
		}
	}
}
